@@ -45,7 +45,6 @@ osi_PrePopulateVCache(struct vcache *avc) {
 
     rw_init(&avc->rwlock, "vcache rwlock", RW_DEFAULT, NULL);
 
-#if defined(AFS_SUN55_ENV)
     /* This is required if the kaio (kernel aynchronous io)
      ** module is installed. Inside the kernel, the function
      ** check_vp( common/os/aio.c) checks to see if the kernel has
@@ -58,11 +57,6 @@ osi_PrePopulateVCache(struct vcache *avc) {
      ** for the time being, we fill up the v_data field with the
      ** vnode pointer itself. */
     avc->v.v_data = (char *)avc;
-#endif /* AFS_SUN55_ENV */
-
-#if defined(AFS_BOZONLOCK_ENV)
-    afs_BozonInit(&avc->pvnLock, avc);
-#endif
 }
 
 void
@@ -74,10 +68,8 @@ osi_PostPopulateVCache(struct vcache *avc) {
     AFSTOV(avc)->v_vfsp = afs_globalVFS;
     vSetType(avc, VREG);
 
-#ifdef AFS_SUN58_ENV
     /* Normally we do this in osi_vnhold when we notice the ref count went from
      * 0 -> 1. But if we just setup or reused a vcache, we set the refcount to
      * 1 directly. So, we must explicitly VFS_HOLD here. */
     VFS_HOLD(afs_globalVFS);
-#endif
 }
