@@ -26,8 +26,6 @@
 #include "afs/auth.h"
 #include "afs/cellconfig.h"
 #include "afs/vice.h"
-#include "afs/kauth.h"
-#include "afs/kautils.h"
 #include "afs/afsutil.h"
 #include "afs/afs_bypasscache.h"
 #include "rx/rx_globals.h"
@@ -94,16 +92,12 @@ afs_lock_t afs_ftf;
 afs_lock_t osi_flplock;
 afs_lock_t osi_fsplock;
 
-#ifndef NETSCAPE_NSAPI
-
 /*
  * Mutex and condition variable used to implement sleep
  */
 pthread_mutex_t usr_sleep_mutex;
 pthread_cond_t usr_sleep_cond;
 
-#endif /* !NETSCAPE_NSAPI */
-
 int call_syscall(long, long, long, long, long, long);
 int fork_syscall(long, long, long, long, long, long);
 
@@ -329,7 +323,7 @@ usr_crcopy(struct usr_ucred *credp)
 {
     struct usr_ucred *newcredp;
 
-    newcredp = (struct usr_ucred *)afs_osi_Alloc(sizeof(struct usr_ucred));
+    newcredp = afs_osi_Alloc(sizeof(struct usr_ucred));
     *newcredp = *credp;
     newcredp->cr_ref = 1;
     return newcredp;
@@ -340,7 +334,7 @@ usr_crget(void)
 {
     struct usr_ucred *newcredp;
 
-    newcredp = (struct usr_ucred *)afs_osi_Alloc(sizeof(struct usr_ucred));
+    newcredp = afs_osi_Alloc(sizeof(struct usr_ucred));
     newcredp->cr_ref = 1;
     return newcredp;
 }
@@ -391,9 +385,7 @@ uafs_InitThread(void)
      * allocate the data block, so pthread_finish can free the buffer
      * when this thread terminates.
      */
-    uptr =
-	(struct usr_user *)malloc(sizeof(struct usr_user) +
-				  sizeof(struct usr_ucred));
+    uptr = malloc(sizeof(struct usr_user) + sizeof(struct usr_ucred));
     usr_assert(uptr != NULL);
     uptr->u_error = 0;
     uptr->u_prio = 0;
@@ -414,11 +406,12 @@ get_user_struct(void)
 {
     struct usr_user *uptr;
     int st;
-    st = usr_getspecific(afs_global_u_key, (void **)&uptr);
+
+    st = usr_getspecific(afs_global_u_key, &uptr);
     usr_assert(st == 0);
     if (uptr == NULL) {
 	uafs_InitThread();
-	st = usr_getspecific(afs_global_u_key, (void **)&uptr);
+	st = usr_getspecific(afs_global_u_key, &uptr);
 	usr_assert(st == 0);
 	usr_assert(uptr != NULL);
     }
@@ -447,7 +440,7 @@ afs_osi_Sleep(void *x)
     }
     index = WAITHASH(x);
     if (osi_waithash_avail == NULL) {
-	waitp = (osi_wait_t *) afs_osi_Alloc(sizeof(osi_wait_t));
+	waitp = afs_osi_Alloc(sizeof(osi_wait_t));
 	usr_cond_init(&waitp->cond);
     } else {
 	waitp = osi_waithash_avail;
@@ -533,7 +526,7 @@ afs_osi_Wait(afs_int32 msec, struct afs_osi_WaitHandle *handle, int intok)
 	}
 	index = WAITHASH((caddr_t) handle);
 	if (osi_waithash_avail == NULL) {
-	    waitp = (osi_wait_t *) afs_osi_Alloc(sizeof(osi_wait_t));
+	    waitp = afs_osi_Alloc(sizeof(osi_wait_t));
 	    usr_cond_init(&waitp->cond);
 	} else {
 	    waitp = osi_waithash_avail;
@@ -660,7 +653,7 @@ osi_UFSOpen(afs_dcache_id_t *ino)
     AFS_ASSERT_GLOCK();
 
     AFS_GUNLOCK();
-    fp = (struct osi_file *)afs_osi_Alloc(sizeof(struct osi_file));
+    fp = afs_osi_Alloc(sizeof(struct osi_file));
     usr_assert(fp != NULL);
 
     usr_assert(ino->ufs);
@@ -1055,14 +1048,12 @@ osi_Init(void)
     afs_global_procp->p_ppid = (pid_t) 1;
     afs_global_procp->p_ucred = afs_global_ucredp;
 
-#ifndef NETSCAPE_NSAPI
     /*
      * Initialize the mutex and condition variable used to implement
      * time sleeps.
      */
     pthread_mutex_init(&usr_sleep_mutex, NULL);
     pthread_cond_init(&usr_sleep_cond, NULL);
-#endif /* !NETSCAPE_NSAPI */
 
     /*
      * Initialize the hash table used for sleep/wakeup
@@ -1387,13 +1378,8 @@ struct syscallThreadArgs {
     long param4;
 };
 
-#ifdef NETSCAPE_NSAPI
-void
-syscallThread(void *argp)
-#else /* NETSCAPE_NSAPI */
 void *
 syscallThread(void *argp)
-#endif				/* NETSCAPE_NSAPI */
 {
     int i;
     struct usr_ucred *crp;
@@ -1654,6 +1640,77 @@ uafs_RPCStatsClearPeer(void)
     return rc;
 }
 
+/*
+ * Lookup the target of a symbolic link
+ * Call VN_HOLD on the output vnode if successful.
+ * Returns zero on success, error code on failure.
+ * If provided, use a path for confirming we are not linked to ourself.
+ *
+ * Note: Caller must hold the AFS global lock.
+ */
+static int
+uafs_LookupLinkPath(struct usr_vnode *vp, struct usr_vnode *parentVp,
+		    char *ppathP, struct usr_vnode **vpp)
+{
+    int code;
+    int len;
+    char *pathP;
+    struct usr_vnode *linkVp;
+    struct usr_uio uio;
+    struct iovec iov[1];
+
+    AFS_ASSERT_GLOCK();
+
+    pathP = afs_osi_Alloc(MAX_OSI_PATH + 1);
+    usr_assert(pathP != NULL);
+
+    /*
+     * set up the uio buffer
+     */
+    iov[0].iov_base = pathP;
+    iov[0].iov_len = MAX_OSI_PATH + 1;
+    uio.uio_iov = &iov[0];
+    uio.uio_iovcnt = 1;
+    uio.uio_offset = 0;
+    uio.uio_segflg = 0;
+    uio.uio_fmode = FREAD;
+    uio.uio_resid = MAX_OSI_PATH + 1;
+
+    /*
+     * Read the link data
+     */
+    code = afs_readlink(VTOAFS(vp), &uio, get_user_struct()->u_cred);
+    if (code) {
+	afs_osi_Free(pathP, MAX_OSI_PATH + 1);
+	return code;
+    }
+    len = MAX_OSI_PATH + 1 - uio.uio_resid;
+    pathP[len] = '\0';
+
+    /* are we linked to ourname or ./ourname? ELOOP */
+    if (ppathP) {
+	if ((strcmp(pathP, ppathP) == 0) ||
+	    ((pathP[0] == '.') &&
+	     (pathP[1] == '/') &&
+	     (strcmp(&(pathP[2]), ppathP) == 0))) {
+	    return ELOOP;
+	}
+    }
+
+    /*
+     * Find the target of the symbolic link
+     */
+    code = uafs_LookupName(pathP, parentVp, &linkVp, 1, 0);
+    if (code) {
+	afs_osi_Free(pathP, MAX_OSI_PATH + 1);
+	return code;
+    }
+
+    afs_osi_Free(pathP, MAX_OSI_PATH + 1);
+    *vpp = linkVp;
+    return 0;
+}
+
 /*
  * Lookup a file or directory given its path.
  * Call VN_HOLD on the output vnode if successful.
@@ -1770,7 +1827,7 @@ uafs_LookupName(char *path, struct usr_vnode *parentVp,
 		    afs_osi_Free(tmpPath, strlen(path) + 1);
 		    return code;
 		}
-		code = uafs_LookupLink(nextVp, vp, &linkVp);
+		code = uafs_LookupLinkPath(nextVp, vp, NULL, &linkVp);
 		if (code) {
 		    VN_RELE(vp);
 		    VN_RELE(nextVp);
@@ -1801,64 +1858,11 @@ uafs_LookupName(char *path, struct usr_vnode *parentVp,
     return 0;
 }
 
-/*
- * Lookup the target of a symbolic link
- * Call VN_HOLD on the output vnode if successful.
- * Returns zero on success, error code on failure.
- *
- * Note: Caller must hold the AFS global lock.
- */
 int
 uafs_LookupLink(struct usr_vnode *vp, struct usr_vnode *parentVp,
 		struct usr_vnode **vpp)
 {
-    int code;
-    int len;
-    char *pathP;
-    struct usr_vnode *linkVp;
-    struct usr_uio uio;
-    struct iovec iov[1];
-
-    AFS_ASSERT_GLOCK();
-
-    pathP = afs_osi_Alloc(MAX_OSI_PATH + 1);
-    usr_assert(pathP != NULL);
-
-    /*
-     * set up the uio buffer
-     */
-    iov[0].iov_base = pathP;
-    iov[0].iov_len = MAX_OSI_PATH + 1;
-    uio.uio_iov = &iov[0];
-    uio.uio_iovcnt = 1;
-    uio.uio_offset = 0;
-    uio.uio_segflg = 0;
-    uio.uio_fmode = FREAD;
-    uio.uio_resid = MAX_OSI_PATH + 1;
-
-    /*
-     * Read the link data
-     */
-    code = afs_readlink(VTOAFS(vp), &uio, get_user_struct()->u_cred);
-    if (code) {
-	afs_osi_Free(pathP, MAX_OSI_PATH + 1);
-	return code;
-    }
-    len = MAX_OSI_PATH + 1 - uio.uio_resid;
-    pathP[len] = '\0';
-
-    /*
-     * Find the target of the symbolic link
-     */
-    code = uafs_LookupName(pathP, parentVp, &linkVp, 1, 0);
-    if (code) {
-	afs_osi_Free(pathP, MAX_OSI_PATH + 1);
-	return code;
-    }
-
-    afs_osi_Free(pathP, MAX_OSI_PATH + 1);
-    *vpp = linkVp;
-    return 0;
+    return uafs_LookupLinkPath(vp, parentVp, NULL, vpp);
 }
 
 /*
@@ -2401,11 +2405,10 @@ uafs_pread_nocache_r(int fd, char *buf, int len, off_t offset)
 
     /* these get freed in PrefetchNoCache, so... */
     bparms = afs_osi_Alloc(sizeof(struct nocache_read_request));
-    bparms->areq = afs_osi_Alloc(sizeof(struct vrequest));
 
-    code = afs_InitReq(bparms->areq, get_user_struct()->u_cred);
+    code = afs_CreateReq(&bparms->areq, get_user_struct()->u_cred);
     if (code) {
-	afs_osi_Free(bparms->areq, sizeof(struct vrequest));
+	afs_DestroyReq(bparms->areq);
 	afs_osi_Free(bparms, sizeof(struct nocache_read_request));
 	errno = code;
 	return -1;
@@ -2459,7 +2462,6 @@ uafs_pread_r(int fd, char *buf, int len, off_t offset)
     struct usr_uio uio;
     struct iovec iov[1];
     struct usr_vnode *fileP;
-    struct usr_buf *bufP;
 
     /*
      * Make sure this is an open file
@@ -2485,7 +2487,7 @@ uafs_pread_r(int fd, char *buf, int len, off_t offset)
     /*
      * do the read
      */
-    code = afs_read(VTOAFS(fileP), &uio, get_user_struct()->u_cred, 0, &bufP, 0);
+    code = afs_read(VTOAFS(fileP), &uio, get_user_struct()->u_cred, 0);
     if (code) {
 	errno = code;
 	return -1;
@@ -3050,7 +3052,7 @@ uafs_symlink_r(char *target, char *source)
     attrs.va_uid = afs_cr_uid(get_user_struct()->u_cred);
     attrs.va_gid = afs_cr_gid(get_user_struct()->u_cred);
     code = afs_symlink(VTOAFS(dirP), nameP, &attrs, target, NULL,
-    		       get_user_struct()->u_cred);
+		       get_user_struct()->u_cred);
     VN_RELE(dirP);
     if (code != 0) {
 	errno = code;
@@ -3401,9 +3403,8 @@ uafs_opendir_r(char *path)
     /*
      * Set up the directory structures
      */
-    dirp =
-	(usr_DIR *) afs_osi_Alloc(sizeof(usr_DIR) + USR_DIRSIZE +
-				  sizeof(struct usr_dirent));
+    dirp = afs_osi_Alloc(sizeof(usr_DIR) + USR_DIRSIZE +
+			 sizeof(struct usr_dirent));
     usr_assert(dirp != NULL);
     dirp->dd_buf = (char *)(dirp + 1);
     dirp->dd_fd = fd;
@@ -3600,34 +3601,6 @@ uafs_closedir_r(usr_DIR * dirp)
     return rc;
 }
 
-/*
- * Do AFS authentication
- */
-int
-uafs_klog(char *user, char *cell, char *passwd, char **reason)
-{
-    int code;
-    afs_int32 password_expires = -1;
-
-    usr_mutex_lock(&osi_authenticate_lock);
-    code =
-	ka_UserAuthenticateGeneral(KA_USERAUTH_VERSION +
-				   KA_USERAUTH_DOSETPAG2, user, NULL, cell,
-				   passwd, 0, &password_expires, 0, reason);
-    usr_mutex_unlock(&osi_authenticate_lock);
-    return code;
-}
-
-int
-uafs_klog_r(char *user, char *cell, char *passwd, char **reason)
-{
-    int retval;
-    AFS_GUNLOCK();
-    retval = uafs_klog(user, cell, passwd, reason);
-    AFS_GLOCK();
-    return retval;
-}
-
 /*
  * Destroy AFS credentials from the kernel cache
  */
@@ -3691,25 +3664,6 @@ uafs_afsPathName(char *path)
     return NULL;
 }
 
-/*
- * uafs_klog_nopag
- * klog but don't allocate a new pag
- */
-int
-uafs_klog_nopag(char *user, char *cell, char *passwd, char **reason)
-{
-    int code;
-    afs_int32 password_expires = -1;
-
-    usr_mutex_lock(&osi_authenticate_lock);
-    code = ka_UserAuthenticateGeneral(KA_USERAUTH_VERSION
-				      /*+KA_USERAUTH_DOSETPAG2 */ , user,
-				      NULL, cell, passwd, 0,
-				      &password_expires, 0, reason);
-    usr_mutex_unlock(&osi_authenticate_lock);
-    return code;
-}
-
 /*
  * uafs_getcellstatus
  * get the cell status
@@ -3746,13 +3700,12 @@ uafs_getvolquota(char *path, afs_int32 * BlocksInUse, afs_int32 * MaxQuota)
 {
     int rc;
     struct afs_ioctl iob;
-    VolumeStatus *status;
-    char buf[1024];
+    VolumeStatus status;
 
     iob.in = 0;
     iob.in_size = 0;
-    iob.out = buf;
-    iob.out_size = 1024;
+    iob.out = (char *)&status;
+    iob.out_size = sizeof status;
 
     rc = call_syscall(AFSCALL_PIOCTL, (long)path, _VICEIOCTL(4), (long)&iob,
 		      0, 0);
@@ -3762,9 +3715,8 @@ uafs_getvolquota(char *path, afs_int32 * BlocksInUse, afs_int32 * MaxQuota)
 	return -1;
     }
 
-    status = (VolumeStatus *) buf;
-    *BlocksInUse = status->BlocksInUse;
-    *MaxQuota = status->MaxQuota;
+    *BlocksInUse = status.BlocksInUse;
+    *MaxQuota = status.MaxQuota;
     return 0;
 }
 
@@ -3777,18 +3729,15 @@ uafs_setvolquota(char *path, afs_int32 MaxQuota)
 {
     int rc;
     struct afs_ioctl iob;
-    VolumeStatus *status;
-    char buf[1024];
+    VolumeStatus status = { 0 };
 
-    iob.in = buf;
-    iob.in_size = 1024;
+    iob.in = (char *)&status;
+    iob.in_size = sizeof status;
     iob.out = 0;
     iob.out_size = 0;
 
-    memset(buf, 0, sizeof(VolumeStatus));
-    status = (VolumeStatus *) buf;
-    status->MaxQuota = MaxQuota;
-    status->MinQuota = -1;
+    status.MaxQuota = MaxQuota;
+    status.MinQuota = -1;
 
     rc = call_syscall(AFSCALL_PIOCTL, (long)path, _VICEIOCTL(5), (long)&iob,
 		      0, 0);
