@@ -53,11 +53,11 @@ afs_CopyOutAttrs(struct vcache *avc, struct vattr *attrs)
     int fakedir = 0;
 
     AFS_STATCNT(afs_CopyOutAttrs);
-    if (afs_fakestat_enable && avc->mvstat == 1)
+    if (afs_fakestat_enable && avc->mvstat == AFS_MVSTAT_MTPT)
 	fakedir = 1;
     attrs->va_type = fakedir ? VDIR : vType(avc);
-#if defined(AFS_SGI_ENV) || defined(AFS_AIX32_ENV) || defined(AFS_SUN5_ENV)
-    attrs->va_mode = fakedir ? 0755 : (mode_t) (avc->f.m.Mode & 0xffff);
+#if defined(AFS_SGI_ENV) || defined(AFS_AIX32_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_DARWIN_ENV)
+    attrs->va_mode = fakedir ? S_IFDIR | 0755 : (mode_t) (avc->f.m.Mode & 0xffff);
 #else
     attrs->va_mode = fakedir ? VDIR | 0755 : avc->f.m.Mode;
 #endif
@@ -92,7 +92,7 @@ afs_CopyOutAttrs(struct vcache *avc, struct vattr *attrs)
 #endif /* AFS_DARWIN_ENV */
     attrs->va_uid = fakedir ? 0 : avc->f.m.Owner;
     attrs->va_gid = fakedir ? 0 : avc->f.m.Group;	/* yeah! */
-#if defined(AFS_SUN56_ENV)
+#if defined(AFS_SUN5_ENV)
     attrs->va_fsid = avc->v.v_vfsp->vfs_fsid.val[0];
 #elif defined(AFS_DARWIN80_ENV)
     VATTR_RETURN(attrs, va_fsid, vfs_statfs(vnode_mount(AFSTOV(avc)))->f_fsid.val[0]);
@@ -101,7 +101,7 @@ afs_CopyOutAttrs(struct vcache *avc, struct vattr *attrs)
 #else /* ! AFS_DARWIN_ENV */
     attrs->va_fsid = 1;
 #endif 
-    if (avc->mvstat == 2) {
+    if (avc->mvstat == AFS_MVSTAT_ROOT) {
 	tvp = afs_GetVolume(&avc->f.fid, 0, READ_LOCK);
 	/* The mount point's vnode. */
 	if (tvp) {
@@ -205,7 +205,7 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
     afs_Trace2(afs_iclSetp, CM_TRACE_GETATTR, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->f.m.Length));
 
-    if (afs_fakestat_enable && avc->mvstat == 1) {
+    if (afs_fakestat_enable && avc->mvstat == AFS_MVSTAT_MTPT) {
 	struct afs_fakestat_state fakestat;
 	struct vrequest *ureq = NULL;
 
@@ -241,12 +241,10 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
 
     AFS_DISCON_LOCK();
 
-#ifdef AFS_BOZONLOCK_ENV
-    afs_BozonLock(&avc->pvnLock, avc);
-#endif
-
-    if (afs_shuttingdown != AFS_RUNNING)
+    if (afs_shuttingdown != AFS_RUNNING) {
+	AFS_DISCON_UNLOCK();
 	return EIO;
+    }
 
     if (!(avc->f.states & CStatd)) {
 	if (!(code = afs_CreateReq(&treq, acred))) {
@@ -256,13 +254,11 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
     } else
 	code = 0;
 
-#ifdef AFS_BOZONLOCK_ENV
+#if defined(AFS_SUN5_ENV)
     if (code == 0)
 	osi_FlushPages(avc, acred);
-    afs_BozonUnlock(&avc->pvnLock, avc);
 #endif
 
-
     if (code == 0) {
 	osi_FlushText(avc);	/* only needed to flush text if text locked last time */
 	code = afs_CopyOutAttrs(avc, attrs);
@@ -309,8 +305,10 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, afs_ucred_t *acred)
 		    }
 #else
 		    if (
-#ifdef AFS_DARWIN_ENV		    
+#if defined(AFS_DARWIN_ENV)
 			vnode_isvroot(AFSTOV(avc))
+#elif defined(AFS_NBSD50_ENV)
+			AFSTOV(avc)->v_vflag & VV_ROOT
 #else
 			AFSTOV(avc)->v_flag & VROOT
 #endif
@@ -536,9 +534,6 @@ afs_setattr(OSI_VC_DECL(avc), struct vattr *attrs,
 
     afs_VAttrToAS(avc, attrs, &astat);	/* interpret request */
     code = 0;
-#ifdef AFS_BOZONLOCK_ENV
-    afs_BozonLock(&avc->pvnLock, avc);
-#endif
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (AFS_NFSXLATORREQ(acred)) {
 	avc->execsOrWriters++;
@@ -616,13 +611,8 @@ afs_setattr(OSI_VC_DECL(avc), struct vattr *attrs,
 	    ReleaseSharedLock(&avc->lock);	/* release lock */
         }
         if (code) {
-	    ObtainWriteLock(&afs_xcbhash, 487);
-	    afs_DequeueCallback(avc);
-	    avc->f.states &= ~CStatd;
-	    ReleaseWriteLock(&afs_xcbhash);
-	    if (avc->f.fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
-	        osi_dnlc_purgedp(avc);
 	    /* error?  erase any changes we made to vcache entry */
+	    afs_StaleVCache(avc);
         }
     } else {
 	ObtainSharedLock(&avc->lock, 712);
@@ -636,9 +626,6 @@ afs_setattr(OSI_VC_DECL(avc), struct vattr *attrs,
 	avc->execsOrWriters--;
     }
 #endif
-#ifdef AFS_BOZONLOCK_ENV
-    afs_BozonUnlock(&avc->pvnLock, avc);
-#endif
 #if defined(AFS_SGI_ENV)
     AFS_RWUNLOCK((vnode_t *) avc, VRWLOCK_WRITE);
 #endif
