@@ -19,7 +19,7 @@
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 #include "afs/afs_osi.h"
-#include "afs/afs_md5.h"
+#include "hcrypto/md5.h"
 
 /* Local variables. */
 afs_rwlock_t afs_xcell;		/* Export for cmdebug peeking at locks */
@@ -834,6 +834,24 @@ afs_GetPrimaryCell(afs_int32 locktype)
     return afs_GetCellByName(afs_thiscell, locktype);
 }
 
+/*!
+ * Return number of the primary cell.
+ * \return
+ *    Cell number, or 0 if primary cell not found
+ */
+afs_int32
+afs_GetPrimaryCellNum(void)
+{
+    struct cell *cell;
+    afs_int32 cellNum = 0;
+    cell = afs_GetPrimaryCell(READ_LOCK);
+    if (cell) {
+	cellNum = cell->cellNum;
+	afs_PutCell(cell, READ_LOCK);
+    }
+    return cellNum;
+}
+
 /*!
  * Returns true if the given cell is the primary cell.
  * \param cell
@@ -909,6 +927,7 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
 {
     struct cell *tc, *tcl = 0;
     afs_int32 i, newc = 0, code = 0;
+    struct md5 m;
 
     AFS_STATCNT(afs_NewCell);
 
@@ -924,7 +943,9 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
 	tc->cellName = afs_strdup(acellName);
 	tc->fsport = AFS_FSPORT;
 	tc->vlport = AFS_VLPORT;
-	AFS_MD5_String(tc->cellHandle, tc->cellName, strlen(tc->cellName));
+	MD5_Init(&m);
+	MD5_Update(&m, tc->cellName, strlen(tc->cellName));
+	MD5_Final(tc->cellHandle, &m);
 	AFS_RWLOCK_INIT(&tc->lock, "cell lock");
 	newc = 1;
 	aflags |= CNoSUID;
