@@ -35,34 +35,16 @@
  */
 
 #include <afsconfig.h>
-
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_MEMORY_H
-#include <memory.h>
-#endif
-#include <errno.h>
-
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include <sys/param.h>
-#include <sys/errno.h>
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <pwd.h>
-
+#include <afs/param.h>
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <ctype.h>
+
+#include <afs/ktc.h>
+#include <afs/token.h>
+
 #define KERBEROS_APPLE_DEPRECATED(x)
 #include <krb5.h>
 #ifdef HAVE_COM_ERR_H
@@ -79,14 +61,10 @@
 #include <afs/com_err.h>
 #endif
 
-#include <afs/param.h>
 #ifdef AFS_SUN5_ENV
 #include <sys/ioccom.h>
 #endif
 
-/* Prevent inclusion of des.h to avoid conflicts with des types */
-#define NO_DES_H_INCLUDE
-
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/vice.h>
@@ -96,7 +74,6 @@
 #include <afs/pterror.h>
 #include <afs/dirpath.h>
 #include <afs/afsutil.h>
-#include <afs/akimpersonate.h>
 
 #include "aklog.h"
 #include "linked_list.h"
@@ -125,10 +102,6 @@
 #define AKLOG_BADPATH 6
 #define AKLOG_MISC 7
 
-#ifndef NULL
-#define NULL 0
-#endif
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -194,6 +167,60 @@ static int get_user_realm(krb5_context, char **);
 #error "Must have either krb5_princ_size or krb5_principal_get_comp_string"
 #endif
 
+#if !defined(HAVE_KRB5_ENCRYPT_TKT_PART) && defined(HAVE_ENCODE_KRB5_ENC_TKT_PART) && defined(HAVE_KRB5_C_ENCRYPT)
+extern krb5_error_code encode_krb5_enc_tkt_part (const krb5_enc_tkt_part *rep,
+						 krb5_data **code);
+
+krb5_error_code
+krb5_encrypt_tkt_part(krb5_context context,
+		      const krb5_keyblock *key,
+		      krb5_ticket *ticket)
+{
+    krb5_data *data = 0;
+    int code;
+    size_t enclen;
+
+    if ((code = encode_krb5_enc_tkt_part(ticket->enc_part2, &data)))
+	goto Done;
+    if ((code = krb5_c_encrypt_length(context, key->enctype,
+				      data->length, &enclen)))
+	goto Done;
+    ticket->enc_part.ciphertext.length = enclen;
+    if (!(ticket->enc_part.ciphertext.data = malloc(enclen))) {
+	code = ENOMEM;
+	goto Done;
+    }
+    if ((code = krb5_c_encrypt(context, key, KRB5_KEYUSAGE_KDC_REP_TICKET,
+			       0, data, &ticket->enc_part))) {
+	free(ticket->enc_part.ciphertext.data);
+	ticket->enc_part.ciphertext.data = 0;
+    }
+Done:
+    if (data) {
+	if (data->data)
+	    free(data->data);
+	free(data);
+    }
+    return code;
+}
+#endif
+
+#if defined(HAVE_KRB5_CREDS_KEYBLOCK)
+
+#define get_cred_keydata(c) c->keyblock.contents
+#define get_cred_keylen(c) c->keyblock.length
+#define get_creds_enctype(c) c->keyblock.enctype
+
+#elif defined(HAVE_KRB5_CREDS_SESSION)
+
+#define get_cred_keydata(c) c->session.keyvalue.data
+#define get_cred_keylen(c) c->session.keyvalue.length
+#define get_creds_enctype(c) c->session.keytype
+
+#else
+#error "Must have either keyblock or session member of krb5_creds"
+#endif
+
 /* MITKerberosShim logs but returns success */
 #if !defined(HAVE_KRB5_524_CONV_PRINCIPAL) || defined(AFS_DARWIN110_ENV) || (!defined(HAVE_KRB5_524_CONVERT_CREDS) && !defined(HAVE_KRB524_CONVERT_CREDS_KDC))
 #define HAVE_NO_KRB5_524
@@ -288,8 +315,11 @@ redirect_errors(const char *who, afs_int32 code, const char *fmt, va_list ap)
 	if (strncmp(str, "unknown", strlen("unknown")) == 0) {
 #ifdef HAVE_KRB5_SVC_GET_MSG
 	    krb5_svc_get_msg(code,&str);
-#elif defined(HAVE_ERROR_MESSAGE)
-	    str = error_message(code);
+#elif defined(HAVE_KRB5_GET_ERROR_MESSAGE)
+	    krb5_context context;
+	    krb5_init_context(&context);
+	    str = krb5_get_error_message(context, code);
+	    krb5_free_context(context);
 #else
 	    ; /* IRIX apparently has neither: use the string we have */
 #endif
@@ -322,7 +352,7 @@ copy_cellinfo(cellinfo_t *cellinfo)
 {
     cellinfo_t *new_cellinfo;
 
-    if ((new_cellinfo = (cellinfo_t *)malloc(sizeof(cellinfo_t))))
+    if ((new_cellinfo = malloc(sizeof(cellinfo_t))))
 	memcpy(new_cellinfo, cellinfo, sizeof(cellinfo_t));
 
     return ((char *)new_cellinfo);
@@ -330,7 +360,8 @@ copy_cellinfo(cellinfo_t *cellinfo)
 
 
 static int
-get_cellconfig(char *cell, struct afsconf_cell *cellconfig, char **local_cell)
+get_cellconfig(const char *config, char *cell,
+	       struct afsconf_cell *cellconfig, char **local_cell)
 {
     int status = AKLOG_SUCCESS;
     struct afsconf_dir *configdir;
@@ -344,10 +375,10 @@ get_cellconfig(char *cell, struct afsconf_cell *cellconfig, char **local_cell)
 	exit(AKLOG_AFS);
     }
 
-    if (!(configdir = afsconf_Open(AFSDIR_CLIENT_ETC_DIRPATH))) {
+    if (!(configdir = afsconf_Open(config))) {
 	fprintf(stderr,
 		"%s: can't get afs configuration (afsconf_Open(%s))\n",
-		progname, AFSDIR_CLIENT_ETC_DIRPATH);
+		progname, config);
 	exit(AKLOG_AFS);
     }
 
@@ -632,14 +663,14 @@ out:
  */
 static int
 rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
-			 struct ktc_token **tokenPtr, char **userPtr) {
+			 struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     char username[BUFSIZ]="";
-    struct ktc_token *token;
+    struct ktc_token token;
+    int status;
 #ifdef HAVE_NO_KRB5_524
     char *p;
     int len;
 #else
-    int status;
     char k4name[ANAME_SZ];
     char k4inst[INST_SZ];
     char k4realm[REALM_SZ];
@@ -685,26 +716,26 @@ rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
     }
 #endif
 
-    token = malloc(sizeof(struct ktc_token));
-    if (token == NULL)
-	return ENOMEM;
+    memset(&token, 0, sizeof(struct ktc_token));
 
-    memset(token, 0, sizeof(struct ktc_token));
-
-    token->kvno = RXKAD_TKT_TYPE_KERBEROS_V5;
-    token->startTime = v5cred->times.starttime;;
-    token->endTime = v5cred->times.endtime;
+    token.kvno = RXKAD_TKT_TYPE_KERBEROS_V5;
+    token.startTime = v5cred->times.starttime;;
+    token.endTime = v5cred->times.endtime;
     if (tkt_DeriveDesKey(get_creds_enctype(v5cred), inkey, inkey_sz,
-			 &token->sessionKey) != 0) {
-	free(token);
+			 &token.sessionKey) != 0) {
 	return RXKADBADKEY;
     }
-    token->ticketLen = v5cred->ticket.length;
-    memcpy(token->ticket, v5cred->ticket.data, token->ticketLen);
+    token.ticketLen = v5cred->ticket.length;
+    memcpy(token.ticket, v5cred->ticket.data, token.ticketLen);
+
+    status = token_importRxkadViceId(tokenPtr, &token, 0);
+    if (status) {
+	return status;
+    }
 
-    *tokenPtr = token;
     if (username[0] != '\0')
 	*userPtr = strdup(username);
+
     return 0;
 }
 
@@ -723,7 +754,8 @@ rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
  * 	structure which should be freed by the caller.
  * @param[out[ userPtr
  * 	A string containing the principal of the user to whom the token was
- * 	issued. This is a malloc'd block which should be freed by the caller.
+ * 	issued. This is a malloc'd block which should be freed by the caller,
+ *      if set.
  *
  * @returns
  * 	0 on success, an error value upon failure
@@ -732,7 +764,7 @@ rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
 #ifdef HAVE_NO_KRB5_524
 static int
 rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
-			  struct ktc_token **tokenPtr, char **userPtr) {
+			  struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     *tokenPtr = NULL;
     *userPtr = NULL;
 
@@ -741,10 +773,10 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
 #else
 static int
 rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
-			  struct ktc_token **tokenPtr, char **userPtr) {
+			  struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     CREDENTIALS cred;
     char username[BUFSIZ];
-    struct ktc_token *token;
+    struct ktc_token token;
     int status;
 
     *tokenPtr = NULL;
@@ -766,11 +798,10 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
 	strcat (username, cred.pinst);
     }
 
-    token = malloc(sizeof(struct ktc_token));
-    memset(token, 0, sizeof(struct ktc_token));
+    memset(&token, 0, sizeof(struct ktc_token));
 
-    token->kvno = cred.kvno;
-    token->startTime = cred.issue_date;
+    token.kvno = cred.kvno;
+    token.startTime = cred.issue_date;
     /*
      * It seems silly to go through a bunch of contortions to
      * extract the expiration time, when the v5 credentials already
@@ -779,12 +810,16 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
      * Note that this isn't a security hole, as the expiration time
      * is also contained in the encrypted token
      */
-    token->endTime = v5cred->times.endtime;
-    memcpy(&token->sessionKey, cred.session, 8);
-    token->ticketLen = cred.ticket_st.length;
-    memcpy(token->ticket, cred.ticket_st.dat, token->ticketLen);
+    token.endTime = v5cred->times.endtime;
+    memcpy(&token.sessionKey, cred.session, 8);
+    token.ticketLen = cred.ticket_st.length;
+    memcpy(token.ticket, cred.ticket_st.dat, token.ticketLen);
+
+    status = token_importRxkadViceId(tokenPtr, &token, 0);
+    if (status) {
+	return status;
+    }
 
-    *tokenPtr = token;
     *userPtr = strdup(username);
 
     return 0;
@@ -808,7 +843,7 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
  * @parma[out] authuser
  * 	A string containing the principal of the user to whom the token was
  * 	issued. This is a malloc'd block which should be freed by the caller,
- *	if set.
+ *      if set.
  * @param[out] foreign
  * 	Whether the user is considered as 'foreign' to the realm of the cell.
  *
@@ -817,12 +852,11 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
  */
 static int
 rxkad_get_token(krb5_context context, struct afsconf_cell *cell, char *realm,
-		struct ktc_token **token, char **authuser, int *foreign) {
+		struct ktc_tokenUnion **token, char **authuser, int *foreign) {
     krb5_creds *v5cred;
     char *realmUsed = NULL;
     char *username = NULL;
     int status;
-    size_t len;
 
     *token = NULL;
     *authuser = NULL;
@@ -847,9 +881,12 @@ rxkad_get_token(krb5_context context, struct afsconf_cell *cell, char *realm,
 	username = NULL;
 	*foreign = 0;
     } else {
-	len = strlen(username)+strlen(realmUsed)+2;
-	*authuser = malloc(len);
-	afs_snprintf(*authuser, len, "%s@%s", username, realmUsed);
+	if (asprintf(authuser, "%s@%s", username, realmUsed) < 0) {
+	    fprintf(stderr, "%s: Out of memory building PTS name\n", progname);
+	    *authuser = NULL;
+	    status = AKLOG_MISC;
+	    goto out;
+	}
 	*foreign = 1;
     }
 
@@ -862,70 +899,14 @@ out:
     return status;
 }
 
-static int
-get_kernel_token(struct afsconf_cell *cell, struct ktc_token **tokenPtr) {
-    struct ktc_principal client, server;
-    struct ktc_token *token;
-    int ret;
-
-    *tokenPtr = NULL;
-
-    strncpy(server.name, AFSKEY, MAXKTCNAMELEN - 1);
-    strncpy(server.instance, AFSINST, MAXKTCNAMELEN - 1);
-    strncpy(server.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    token = malloc(sizeof(struct ktc_token));
-    if (token == NULL)
-	return ENOMEM;
-
-    memset(token, 0, sizeof(struct ktc_token));
-
-    ret = ktc_GetToken(&server, token, sizeof(struct ktc_token), &client);
-    if (ret) {
-	free(token);
-	return ret;
-    }
-
-    *tokenPtr = token;
-    return 0;
-}
-
-static int
-set_kernel_token(struct afsconf_cell *cell, char *username,
-		 struct ktc_token *token, int setpag)
-{
-    struct ktc_principal client, server;
-
-    strncpy(client.name,
-	    username ? username : "",
-	    MAXKTCNAMELEN - 1);
-    strcpy(client.instance, "");
-    strncpy(client.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    strncpy(server.name, AFSKEY, MAXKTCNAMELEN - 1);
-    strncpy(server.instance, AFSINST, MAXKTCNAMELEN - 1);
-    strncpy(server.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    return ktc_SetToken(&server, token, &client, setpag);
-}
-
-static int
-tokens_equal(struct ktc_token *tokenA, struct ktc_token *tokenB) {
-    return (tokenA != NULL && tokenB != NULL &&
-	    tokenA->kvno == tokenB->kvno &&
-	    tokenA->ticketLen == tokenB->ticketLen &&
-	    !memcmp(&tokenA->sessionKey, &tokenB->sessionKey,
-		    sizeof(tokenA->sessionKey)) &&
-	    !memcmp(tokenA->ticket, tokenB->ticket, tokenA->ticketLen));
-}
-
-/*
+/* 
  * Log to a cell.  If the cell has already been logged to, return without
  * doing anything.  Otherwise, log to it and mark that it has been logged
  * to.
  */
 static int
-auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
+auth_to_cell(krb5_context context, const char *config, 
+	     char *cell, char *realm, char **linkedcell)
 {
     int status = AKLOG_SUCCESS;
     int isForeign = 0;
@@ -933,12 +914,13 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
     afs_int32 viceId;		/* AFS uid of user */
 
     char *local_cell = NULL;
-    struct ktc_token *token;
-    struct ktc_token *btoken;
+    struct ktc_tokenUnion *rxkadToken = NULL;
+    struct ktc_setTokenData *token;
+    struct ktc_setTokenData *btoken = NULL;
     struct afsconf_cell cellconf;
 
     /* NULL or empty cell returns information on local cell */
-    if ((status = get_cellconfig(cell, &cellconf, &local_cell)))
+    if ((status = get_cellconfig(config, cell, &cellconf, &local_cell)))
 	return(status);
 
     if (linkedcell != NULL) {
@@ -990,20 +972,38 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	afs_dprintf("Authenticating to cell %s (server %s).\n", cellconf.name,
 		cellconf.hostName[0]);
 
-	status = rxkad_get_token(context, &cellconf, realm, &token,
+	token = token_buildTokenJar(cellconf.name);
+	if (token == NULL) {
+	    status = ENOMEM;
+	    goto out;
+	}
+
+	status = rxkad_get_token(context, &cellconf, realm, &rxkadToken,
 				 &username, &isForeign);
 	if (status)
-	    return status;
+	    goto out;
 
+	/* We need to keep the token structure around so that we can stick
+	 * the viceId into it (once we know it) */
+	status = token_addToken(token, rxkadToken);
+	if (status) {
+	    afs_dprintf("Add Token failed with %d", status);
+	    goto out;
+	}
 
 	if (!force &&
-	    !get_kernel_token(&cellconf, &btoken) &&
-	    tokens_equal(token, btoken)) {
+	    ktc_GetTokenEx(cellconf.name, &btoken) == 0 &&
+	    token_SetsEquivalent(token, btoken)) {
+
+	    token_FreeSet(&btoken);
 	    afs_dprintf("Identical tokens already exist; skipping.\n");
 	    status = AKLOG_SUCCESS;
 	    goto out;
 	}
 
+	if (btoken)
+	    token_FreeSet(&btoken);
+
 #ifdef FORCE_NOPRDB
 	noprdb = 1;
 #endif
@@ -1038,7 +1038,7 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 			username, cellconf.name);
 		viceId = 0;
 
-		status = set_kernel_token(&cellconf, username, token, 0);
+		status = ktc_SetTokenEx(token);
 		if (status) {
 		    afs_com_err(progname, status,
 				"while obtaining tokens for cell %s",
@@ -1049,7 +1049,7 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 		/*
 		 * In case you're wondering, we don't need to change the
 		 * filename here because we're still connecting to the
-		 * same cell -- we're just using a different authentication
+		 * same cell -- we're just using a different authenticat ion
 		 * level
 		 */
 
@@ -1071,28 +1071,18 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	    }
 #endif /* ALLOW_REGISTER */
 
-	    /*
-	     * This is a crock, but it is Transarc's crock, so we have to play
-	     * along in order to get the functionality.  The way the afs id is
-	     * stored is as a string in the username field of the token.
-	     * Contrary to what you may think by looking at the code for
-	     * tokens, this hack (AFS ID %d) will not work if you change %d
-	     * to something else.
-	     */
-
 	    if ((status == 0) && (viceId != ANONYMOUSID)) {
-		free(username);
-		if (afs_asprintf(&username, "AFS ID %d", (int) viceId) < 0) {
-		    status = ENOMEM;
-		    username = NULL;
-		    goto out;
+		status = token_setRxkadViceId(rxkadToken, viceId);
+		if (status) {
+		    fprintf(stderr, "Error %d setting rxkad ViceId\n", status);
+		    status = AKLOG_SUCCESS;
+		} else {
+		    token_replaceToken(token, rxkadToken);
 		}
 	    }
 	}
 
 	if (username) {
-	    afs_dprintf("Set username to %s\n", username);
-
 	    afs_dprintf("Setting tokens. %s @ %s\n",
 			username, cellconf.name);
 	} else {
@@ -1107,12 +1097,14 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	 * We don't care about the return value, but need to collect it
 	 * to avoid compiler warnings.
 	 */
-	if (write(2,"",0) < 0) /* dummy write */
-	    ; /* don't care */
+	if (write(2,"",0) < 0) {
+	    /* dummy write, don't care */
+	}
 #endif
-	status = set_kernel_token(&cellconf, username, token, afssetpag);
+	token_setPag(token, afssetpag);
+	status = ktc_SetTokenEx(token);
 	if (status) {
-	    afs_com_err(progname, status, "while obtaining tokens for cell %s",
+	    afs_com_err(progname, status, "while setting tokens for cell %s",
 			cellconf.name);
 	    status = AKLOG_TOKEN;
 	}
@@ -1121,6 +1113,10 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	afs_dprintf("Noauth mode; not authenticating.\n");
 
 out:
+    if (rxkadToken) {
+	token_freeToken(&rxkadToken);
+    }
+
     if (local_cell)
 	free(local_cell);
     if (username)
@@ -1335,7 +1331,7 @@ add_hosts(char *file)
  * every cell it encounters along the way.
  */
 static int
-auth_to_path(krb5_context context, char *path)
+auth_to_path(krb5_context context, const char *config, char *path)
 {
     int status = AKLOG_SUCCESS;
     int auth_status = AKLOG_SUCCESS;
@@ -1385,7 +1381,8 @@ auth_to_path(krb5_context context, char *path)
 		add_hosts(pathtocheck);
 	    if ((endofcell = strchr(mountpoint, VOLMARKER))) {
 		*endofcell = '\0';
-		if ((auth_status = auth_to_cell(context, cell, NULL, NULL))) {
+		auth_status = auth_to_cell(context, config, cell, NULL, NULL);
+		if (auth_status) {
 		    if (status == AKLOG_SUCCESS)
 			status = auth_status;
 		    else if (status != auth_status)
@@ -1473,6 +1470,7 @@ main(int argc, char *argv[])
     linked_list paths;		/* List of paths to log to */
     ll_node *cur_node;
     char *linkedcell;
+    const char *config = AFSDIR_CLIENT_ETC_DIRPATH;
 
     memset(&cellinfo, 0, sizeof(cellinfo));
 
@@ -1503,20 +1501,25 @@ main(int argc, char *argv[])
 	filepath = getenv("KRB5_CONFIG");
 
 	/* only fiddle with KRB5_CONFIG if krb5-weak.conf actually exists */
-	afs_asprintf(&newpath, "%s/krb5-weak.conf", AFSDIR_CLIENT_ETC_DIRPATH);
-	if (access(newpath, R_OK) == 0) {
+	if (asprintf(&newpath, "%s/krb5-weak.conf",
+		     AFSDIR_CLIENT_ETC_DIRPATH) < 0)
+	    newpath = NULL;
+	if (newpath != NULL && access(newpath, R_OK) == 0) {
 	    free(newpath);
 	    newpath = NULL;
-	    afs_asprintf(&newpath, "%s:%s/krb5-weak.conf",
-	                 filepath ? filepath : defaultpath,
-	                 AFSDIR_CLIENT_ETC_DIRPATH);
-	    setenv("KRB5_CONFIG", newpath, 1);
+	    if (asprintf(&newpath, "%s:%s/krb5-weak.conf",
+			 filepath ? filepath : defaultpath,
+			 AFSDIR_CLIENT_ETC_DIRPATH) < 0)
+		newpath = NULL;
+	    else
+		setenv("KRB5_CONFIG", newpath, 1);
 	}
 #endif
 	krb5_init_context(&context);
 
 #if defined(KRB5_PROG_ETYPE_NOSUPP) && !(defined(HAVE_KRB5_ENCTYPE_ENABLE) || defined(HAVE_KRB5_ALLOW_WEAK_CRYPTO))
-	free(newpath);
+	if (newpath)
+	    free(newpath);
 	if (filepath)
 	    setenv("KRB5_CONFIG", filepath, 1);
 	else
@@ -1596,7 +1599,12 @@ main(int argc, char *argv[])
 	    }
 	    else
 		usage();
-
+	else if (strcmp(argv[i], "-config") == 0)
+	    if (++i < argc) {
+		config = argv[i];
+	    }
+	    else
+		usage();
 	else if (argv[i][0] == '-')
 	    usage();
 	else if (!pmode && !cmode) {
@@ -1670,7 +1678,7 @@ main(int argc, char *argv[])
     if ((cells.nelements + paths.nelements) == 0) {
 	struct passwd *pwd;
 
-	status = auth_to_cell(context, NULL, NULL, &linkedcell);
+	status = auth_to_cell(context, config, NULL, NULL, &linkedcell);
 
 	/* If this cell is linked to a DCE cell, and user requested -linked,
 	 * get tokens for both. This is very useful when the AFS cell is
@@ -1679,7 +1687,7 @@ main(int argc, char *argv[])
 
 	if (!status && linked && linkedcell != NULL) {
 	    afs_dprintf("Linked cell: %s\n", linkedcell);
-	    status = auth_to_cell(context, linkedcell, NULL, NULL);
+	    status = auth_to_cell(context, config, linkedcell, NULL, NULL);
 	}
 	if (linkedcell) {
 	    free(linkedcell);
@@ -1713,7 +1721,7 @@ main(int argc, char *argv[])
 
 		    afs_dprintf("Found cell %s in %s.\n", fcell, xlog_path);
 
-		    auth_status = auth_to_cell(context, fcell, NULL, NULL);
+		    auth_status = auth_to_cell(context, config, fcell, NULL, NULL);
 		    if (status == AKLOG_SUCCESS)
 			status = auth_status;
 		    else
@@ -1726,14 +1734,16 @@ main(int argc, char *argv[])
 	/* Log to all cells in the cells list first */
 	for (cur_node = cells.first; cur_node; cur_node = cur_node->next) {
 	    memcpy((char *)&cellinfo, cur_node->data, sizeof(cellinfo));
-	    if ((status = auth_to_cell(context, cellinfo.cell, cellinfo.realm,
-				       &linkedcell)))
+	    status = auth_to_cell(context, config, cellinfo.cell,
+				  cellinfo.realm, &linkedcell);
+	    if (status) {
 		somethingswrong++;
-	    else {
+	    } else {
 		if (linked && linkedcell != NULL) {
 		    afs_dprintf("Linked cell: %s\n", linkedcell);
-		    if ((status = auth_to_cell(context, linkedcell,
-					       cellinfo.realm, NULL)))
+		    status = auth_to_cell(context, config, linkedcell,
+					  cellinfo.realm, NULL);
+		    if (status)
 			somethingswrong++;
 		}
 		if (linkedcell != NULL) {
@@ -1745,7 +1755,8 @@ main(int argc, char *argv[])
 
 	/* Then, log to all paths in the paths list */
 	for (cur_node = paths.first; cur_node; cur_node = cur_node->next) {
-	    if ((status = auth_to_path(context, cur_node->data)))
+	    status = auth_to_path(context, config, cur_node->data);
+	    if (status)
 		somethingswrong++;
 	}
 
@@ -1789,6 +1800,327 @@ isdir(char *path, unsigned char *val)
     }
 }
 
+static krb5_error_code
+get_credv5_akimpersonate(krb5_context context,
+			 char* keytab,
+			 krb5_principal service_principal,
+			 krb5_principal client_principal,
+			 time_t starttime,
+			 time_t endtime,
+			 int *allowed_enctypes,
+			 int *paddress,
+			 krb5_creds** out_creds /* out */ )
+{
+#if defined(USING_HEIMDAL) || (defined(HAVE_ENCODE_KRB5_ENC_TKT) && defined(HAVE_ENCODE_KRB5_TICKET) && defined(HAVE_KRB5_C_ENCRYPT))
+    krb5_error_code code;
+    krb5_keytab kt = 0;
+    krb5_kt_cursor cursor[1];
+    krb5_keytab_entry entry[1];
+    krb5_ccache cc = 0;
+    krb5_creds *creds = 0;
+    krb5_enctype enctype;
+    krb5_kvno kvno;
+    krb5_keyblock session_key[1];
+#if USING_HEIMDAL
+    Ticket ticket_reply[1];
+    EncTicketPart enc_tkt_reply[1];
+    krb5_address address[30];
+    krb5_addresses faddr[1];
+    unsigned int temp_vno[1];
+    time_t temp_time[2];
+#else
+    krb5_ticket ticket_reply[1];
+    krb5_enc_tkt_part enc_tkt_reply[1];
+    krb5_address address[30], *faddr[30];
+    krb5_data * temp;
+#endif
+    int i;
+    static int any_enctype[] = {0};
+    *out_creds = 0;
+    if (!(creds = malloc(sizeof *creds))) {
+        code = ENOMEM;
+        goto cleanup;
+    }
+    if (!allowed_enctypes)
+        allowed_enctypes = any_enctype;
+
+    cc = 0;
+    enctype = 0; /* AKIMPERSONATE_IGNORE_ENCTYPE */
+    kvno = 0; /* AKIMPERSONATE_IGNORE_VNO */
+    memset((char*)creds, 0, sizeof *creds);
+    memset((char*)entry, 0, sizeof *entry);
+    memset((char*)session_key, 0, sizeof *session_key);
+    memset((char*)ticket_reply, 0, sizeof *ticket_reply);
+    memset((char*)enc_tkt_reply, 0, sizeof *enc_tkt_reply);
+    code = krb5_kt_resolve(context, keytab, &kt);
+    if (code) {
+        if (keytab)
+            afs_com_err(progname, code, "while resolving keytab %s", keytab);
+        else
+            afs_com_err(progname, code, "while resolving default keytab");
+        goto cleanup;
+    }
+
+    if (service_principal) {
+        for (i = 0; (enctype = allowed_enctypes[i]) || !i; ++i) {
+	    code = krb5_kt_get_entry(context,
+				     kt,
+				     service_principal,
+				     kvno,
+				     enctype,
+				     entry);
+	    if (!code) {
+		if (allowed_enctypes[i])
+		    deref_keyblock_enctype(session_key) = allowed_enctypes[i];
+		break;
+	    }
+        }
+        if (code) {
+	    afs_com_err(progname, code,"while scanning keytab entries");
+	    goto cleanup;
+        }
+    } else {
+        krb5_keytab_entry new[1];
+        int best = -1;
+        memset(new, 0, sizeof *new);
+        if ((code = krb5_kt_start_seq_get(context, kt, cursor))) {
+            afs_com_err(progname, code, "while starting keytab scan");
+            goto cleanup;
+        }
+        while (!(code = krb5_kt_next_entry(context, kt, new, cursor))) {
+            for (i = 0;
+                    allowed_enctypes[i] && allowed_enctypes[i]
+		     != deref_entry_enctype(new); ++i)
+                ;
+            if ((!i || allowed_enctypes[i]) &&
+		(best < 0 || best > i)) {
+                krb5_free_keytab_entry_contents(context, entry);
+                *entry = *new;
+                memset(new, 0, sizeof *new);
+            } else krb5_free_keytab_entry_contents(context, new);
+        }
+        if ((i = krb5_kt_end_seq_get(context, kt, cursor))) {
+            afs_com_err(progname, i, "while ending keytab scan");
+            code = i;
+            goto cleanup;
+        }
+        if (best < 0) {
+            afs_com_err(progname, code, "while scanning keytab");
+            goto cleanup;
+        }
+        deref_keyblock_enctype(session_key) = deref_entry_enctype(entry);
+    }
+
+    /* Make Ticket */
+
+#if USING_HEIMDAL
+    if ((code = krb5_generate_random_keyblock(context,
+					      deref_keyblock_enctype(session_key), session_key))) {
+        afs_com_err(progname, code, "while making session key");
+        goto cleanup;
+    }
+    enc_tkt_reply->flags.initial = 1;
+    enc_tkt_reply->transited.tr_type = DOMAIN_X500_COMPRESS;
+    enc_tkt_reply->cname = client_principal->name;
+    enc_tkt_reply->crealm = client_principal->realm;
+    enc_tkt_reply->key = *session_key;
+    {
+        static krb5_data empty_string;
+        enc_tkt_reply->transited.contents = empty_string;
+    }
+    enc_tkt_reply->authtime = starttime;
+    enc_tkt_reply->starttime = temp_time;
+    *enc_tkt_reply->starttime = starttime;
+#if 0
+    enc_tkt_reply->renew_till = temp_time + 1;
+    *enc_tkt_reply->renew_till = endtime;
+#endif
+    enc_tkt_reply->endtime = endtime;
+#else
+    if ((code = krb5_c_make_random_key(context,
+				       deref_keyblock_enctype(session_key), session_key))) {
+        afs_com_err(progname, code, "while making session key");
+        goto cleanup;
+    }
+    enc_tkt_reply->magic = KV5M_ENC_TKT_PART;
+#define DATACAST        (unsigned char *)
+    enc_tkt_reply->flags |= TKT_FLG_INITIAL;
+    enc_tkt_reply->transited.tr_type = KRB5_DOMAIN_X500_COMPRESS;
+    enc_tkt_reply->session = session_key;
+    enc_tkt_reply->client = client_principal;
+    {
+        static krb5_data empty_string;
+        enc_tkt_reply->transited.tr_contents = empty_string;
+    }
+    enc_tkt_reply->times.authtime = starttime;
+    enc_tkt_reply->times.starttime = starttime; /* krb524init needs this */
+    enc_tkt_reply->times.endtime = endtime;
+#endif  /* USING_HEIMDAL */
+    /* NB:  We will discard address for now--ignoring caddr field
+       in any case.  MIT branch does what it always did. */
+
+    if (paddress && *paddress) {
+        deref_enc_tkt_addrs(enc_tkt_reply) = faddr;
+#if USING_HEIMDAL
+        faddr->len = 0;
+        faddr->val = address;
+#endif
+        for (i = 0; paddress[i]; ++i) {
+#if USING_HEIMDAL
+            address[i].addr_type = KRB5_ADDRESS_INET;
+            address[i].address.data = (void*)(paddress+i);
+            address[i].address.length = sizeof(paddress[i]);
+#else
+#if !USING_SSL
+            address[i].magic = KV5M_ADDRESS;
+            address[i].addrtype = ADDRTYPE_INET;
+#else
+            address[i].addrtype = AF_INET;
+#endif
+            address[i].contents = (void*)(paddress+i);
+            address[i].length = sizeof(int);
+            faddr[i] = address+i;
+#endif
+        }
+#if USING_HEIMDAL
+        faddr->len = i;
+#else
+        faddr[i] = 0;
+#endif
+    }
+
+#if USING_HEIMDAL
+    ticket_reply->sname = service_principal->name;
+    ticket_reply->realm = service_principal->realm;
+
+    { /* crypto block */
+        krb5_crypto crypto = 0;
+        unsigned char *buf = 0;
+        size_t buf_size, buf_len;
+        char *what;
+
+        ASN1_MALLOC_ENCODE(EncTicketPart, buf, buf_size,
+			   enc_tkt_reply, &buf_len, code);
+        if(code) {
+            afs_com_err(progname, code, "while encoding ticket");
+            goto cleanup;
+        }
+
+        if(buf_len != buf_size) {
+            afs_com_err(progname, code,
+		    "%u != %u while encoding ticket (internal ASN.1 encoder error",
+		    (unsigned int)buf_len, (unsigned int)buf_size);
+            goto cleanup;
+        }
+        what = "krb5_crypto_init";
+        code = krb5_crypto_init(context,
+				&deref_entry_keyblock(entry),
+				deref_entry_enctype(entry),
+				&crypto);
+        if(!code) {
+            what = "krb5_encrypt";
+            code = krb5_encrypt_EncryptedData(context, crypto, KRB5_KU_TICKET,
+					      buf, buf_len, entry->vno, &(ticket_reply->enc_part));
+        }
+        if (buf) free(buf);
+        if (crypto) krb5_crypto_destroy(context, crypto);
+        if(code) {
+            afs_com_err(progname, code, "while %s", what);
+            goto cleanup;
+        }
+    } /* crypto block */
+    ticket_reply->enc_part.etype = deref_entry_enctype(entry);
+    ticket_reply->enc_part.kvno = (void *)temp_vno;
+    *ticket_reply->enc_part.kvno = entry->vno;
+    ticket_reply->tkt_vno = 5;
+#else
+    ticket_reply->server = service_principal;
+    ticket_reply->enc_part2 = enc_tkt_reply;
+    if ((code = krb5_encrypt_tkt_part(context, &deref_entry_keyblock(entry), ticket_reply))) {
+        afs_com_err(progname, code, "while making ticket");
+        goto cleanup;
+    }
+    ticket_reply->enc_part.kvno = entry->vno;
+#endif
+
+    /* Construct Creds */
+
+    if ((code = krb5_copy_principal(context, service_principal,
+				    &creds->server))) {
+        afs_com_err(progname, code, "while copying service principal");
+        goto cleanup;
+    }
+    if ((code = krb5_copy_principal(context, client_principal,
+				    &creds->client))) {
+        afs_com_err(progname, code, "while copying client principal");
+        goto cleanup;
+    }
+    if ((code = krb5_copy_keyblock_contents(context, session_key,
+					    &deref_session_key(creds)))) {
+        afs_com_err(progname, code, "while copying session key");
+        goto cleanup;
+    }
+
+#if USING_HEIMDAL
+    creds->times.authtime = enc_tkt_reply->authtime;
+    creds->times.starttime = *(enc_tkt_reply->starttime);
+    creds->times.endtime = enc_tkt_reply->endtime;
+    creds->times.renew_till = 0; /* *(enc_tkt_reply->renew_till) */
+    creds->flags.b = enc_tkt_reply->flags;
+#else
+    creds->times = enc_tkt_reply->times;
+    creds->ticket_flags = enc_tkt_reply->flags;
+#endif
+    if (!deref_enc_tkt_addrs(enc_tkt_reply))
+        ;
+    else if ((code = krb5_copy_addresses(context,
+					 deref_enc_tkt_addrs(enc_tkt_reply), &creds->addresses))) {
+        afs_com_err(progname, code, "while copying addresses");
+        goto cleanup;
+    }
+
+#if USING_HEIMDAL
+    {
+	size_t creds_tkt_len;
+	ASN1_MALLOC_ENCODE(Ticket, creds->ticket.data, creds->ticket.length,
+			   ticket_reply, &creds_tkt_len, code);
+	if(code) {
+	    afs_com_err(progname, code, "while encoding ticket");
+	    goto cleanup;
+	}
+    }
+#else
+    if ((code = encode_krb5_ticket(ticket_reply, &temp))) {
+	afs_com_err(progname, code, "while encoding ticket");
+	goto cleanup;
+    }
+    creds->ticket = *temp;
+    free(temp);
+#endif
+    /* return creds */
+    *out_creds = creds;
+    creds = 0;
+cleanup:
+    if (deref_enc_data(&ticket_reply->enc_part))
+        free(deref_enc_data(&ticket_reply->enc_part));
+    krb5_free_keytab_entry_contents(context, entry);
+    if (client_principal)
+        krb5_free_principal(context, client_principal);
+    if (service_principal)
+        krb5_free_principal(context, service_principal);
+    if (cc)
+        krb5_cc_close(context, cc);
+    if (kt)
+        krb5_kt_close(context, kt);
+    if (creds) krb5_free_creds(context, creds);
+    krb5_free_keyblock_contents(context, session_key);
+    return code;
+#else
+    return -1;
+#endif
+}
+
+
 static krb5_error_code
 get_credv5(krb5_context context, char *name, char *inst, char *realm,
 	   krb5_creds **creds)
@@ -1805,8 +2137,8 @@ get_credv5(krb5_context context, char *name, char *inst, char *realm,
     if ((r = krb5_build_principal(context, &increds.server,
 				  strlen(realm), realm,
 				  name,
-				  (inst && strlen(inst)) ? inst : (void *) NULL,
-				  (void *) NULL))) {
+				  (inst && strlen(inst)) ? inst : NULL,
+				  NULL))) {
         return r;
     }
 
@@ -1833,12 +2165,17 @@ get_credv5(krb5_context context, char *name, char *inst, char *realm,
 	get_creds_enctype((&increds)) = ENCTYPE_DES_CBC_CRC;
 
     if (keytab) {
+	int allowed_enctypes[] = {
+	    ENCTYPE_DES_CBC_CRC, 0
+	};
+
 	r = get_credv5_akimpersonate(context,
 				     keytab,
 				     increds.server,
 				     increds.client,
-				     0, 0x7fffffff,
-				     NULL,
+				     300, ((~0U)>>1),
+				     allowed_enctypes,
+				     0 /* paddress */,
 				     creds /* out */);
     } else {
 	r = krb5_get_credentials(context, 0, _krb425_ccache, &increds, creds);
