@@ -9,47 +9,32 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <afs/pthread_glock.h>
-#include <sys/types.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <sys/utime.h>
-#include <io.h>
 #include <WINNT/afssw.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <sys/file.h>
-#include <sys/time.h>
+#endif
+
 #include <ctype.h>
-#include <arpa/nameser.h>
+
 #ifdef HAVE_ARPA_NAMESER_COMPAT_H
 #include <arpa/nameser_compat.h>
 #endif
-#include <resolv.h>
-#endif /* AFS_NT40_ENV */
+
+#include <afs/pthread_glock.h>
 #include <afs/afsint.h>
-#include <errno.h>
-#include <ctype.h>
-#include <time.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <rx/rxkad.h>
 #include <rx/rx.h>
 
 #include <afs/afsutil.h>
-#include "cellconfig.h"
 #include "keys.h"
+#include "cellconfig.h"
+#include "internal.h"
+
 #ifdef AFS_NT40_ENV
 #include <cm.h>
 #include <cm_config.h>
@@ -84,12 +69,9 @@ static struct afsconf_servPair serviceTable[] = {
 
 /* Prototypes */
 static int TrimLine(char *abuffer, int abufsize);
-static int IsClientConfigDirectory(const char *path);
 #ifdef AFS_NT40_ENV
 static int GetCellNT(struct afsconf_dir *adir);
 #endif
-static int afsconf_Check(struct afsconf_dir *adir);
-static int afsconf_Touch(struct afsconf_dir *adir);
 static int GetCellUnix(struct afsconf_dir *adir);
 static int afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 				char clones[]);
@@ -99,7 +81,6 @@ static int ParseCellLine(char *aline, char *aname,
 			 char *alname);
 static int afsconf_CloseInternal(struct afsconf_dir *adir);
 static int afsconf_Reopen(struct afsconf_dir *adir);
-static int SaveKeys(struct afsconf_dir *adir);
 
 #ifndef T_AFSDB
 #define T_AFSDB 18		/* per RFC1183 section 1 */
@@ -157,7 +138,7 @@ afsconf_fopen(const char *fname, const char *fmode)
     iop->buffer = malloc(BUFFER);
     if (iop->buffer == NULL) {
 	(void) close(fd);
-	free((void *) iop);
+	free(iop);
 	errno = ENOMEM;
 	return NULL;
     }
@@ -173,8 +154,8 @@ afsconf_fclose(afsconf_FILE *iop)
 	return 0;
     }
     close(iop->_file);
-    free((void *)iop->buffer);
-    free((void *)iop);
+    free(iop->buffer);
+    free(iop);
     return 0;
 }
 
@@ -297,8 +278,8 @@ TrimLine(char *abuffer, int abufsize)
 #else /* AFS_NT40_ENV */
 #define IS_SEP(x) ((x) == '/')
 #endif /* AFS_NT40_ENV */
-static int
-IsClientConfigDirectory(const char *path)
+int
+_afsconf_IsClientConfigDirectory(const char *path)
 {
     const char *cdir = AFSDIR_CLIENT_ETC_DIRPATH;
     int i, cc, pc;
@@ -337,105 +318,118 @@ IsClientConfigDirectory(const char *path)
     return 1;
 }
 
-
-static int
-afsconf_Check(struct afsconf_dir *adir)
-{
-    char tbuffer[256];
 #ifdef AFS_NT40_ENV
+static void
+_afsconf_CellServDBPath(struct afsconf_dir *adir, char **path)
+{
     char *p;
-#endif
+
+    /* NT client CellServDB has different file name than NT server or Unix */
+    if (_afsconf_IsClientConfigDirectory(adir->name)) {
+	if (!afssw_GetClientCellServDBDir(&p)) {
+	    if (asprintf(path, "%s/%s", p, AFSDIR_CELLSERVDB_FILE_NTCLIENT) < 0)
+		*path = NULL;
+	    free(p);
+	} else {
+	    if (asprintf(path, "%s/%s", adir->name,
+			 AFSDIR_CELLSERVDB_FILE_NTCLIENT) < 0)
+		*path = NULL;
+	}
+    } else {
+	if (asprintf(path, "%s/%s", adir->name, AFSDIR_CELLSERVDB_FILE) < 0)
+	    *path = NULL;
+    }
+    return;
+}
+#else
+static void
+_afsconf_CellServDBPath(struct afsconf_dir *adir, char **path)
+{
+    if (asprintf(path, "%s/%s", adir->name, AFSDIR_CELLSERVDB_FILE) < 0)
+	*path = NULL;
+}
+#endif /* AFS_NT40_ENV */
+
+int
+_afsconf_UpToDate(struct afsconf_dir *adir)
+{
+    char *cellservDB;
     struct stat tstat;
-    afs_int32 code;
+    int code;
     time_t now = time(0);
 
     if (adir->timeRead && (adir->timeCheck == now)) {
-	return 0; /* stat no more than once a second */
+	return 1; /* stat no more than once a second */
     }
     adir->timeCheck = now;
 
-#ifdef AFS_NT40_ENV
-    /* NT client CellServDB has different file name than NT server or Unix */
-    if (IsClientConfigDirectory(adir->name)) {
-	if (!afssw_GetClientCellServDBDir(&p)) {
-	    strcompose(tbuffer, sizeof(tbuffer), p, "/",
-		       AFSDIR_CELLSERVDB_FILE_NTCLIENT, NULL);
-	    free(p);
-	} else {
-	    int len;
-	    strncpy(tbuffer, adir->name, sizeof(tbuffer));
-	    len = (int)strlen(tbuffer);
-	    if (tbuffer[len - 1] != '\\' && tbuffer[len - 1] != '/') {
-		strncat(tbuffer, "\\", sizeof(tbuffer));
-	    }
-	    strncat(tbuffer, AFSDIR_CELLSERVDB_FILE_NTCLIENT,
-		    sizeof(tbuffer));
-	    tbuffer[sizeof(tbuffer) - 1] = '\0';
-	}
-    } else {
-	strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE,
-		   NULL);
-    }
-#else
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE, NULL);
-#endif /* AFS_NT40_ENV */
-
-    code = stat(tbuffer, &tstat);
-    if (code < 0) {
-	return code;
-    }
-    /* did file change? */
-    if (tstat.st_mtime == adir->timeRead) {
+    _afsconf_CellServDBPath(adir, &cellservDB);
+    if (cellservDB == NULL)
 	return 0;
-    }
+
+    code = stat(cellservDB, &tstat);
+    free(cellservDB);
+    if (code < 0)
+	return 0; /* Can't throw the error, so just say we're not up to date */
+
+    /* did file change? */
+    if (tstat.st_mtime == adir->timeRead)
+	return 1;
+
+    /* otherwise file has changed */
+    return 0;
+}
+
+int
+afsconf_UpToDate(void *rock)
+{
+    int code;
+
+    LOCK_GLOBAL_MUTEX;
+    code = _afsconf_UpToDate(rock);
+    UNLOCK_GLOBAL_MUTEX;
+
+    return code;
+}
+
+int
+_afsconf_Check(struct afsconf_dir *adir)
+{
+    /* did configuration change? */
+    if (_afsconf_UpToDate(adir))
+	return 0;
+
     /* otherwise file has changed, so reopen it */
     return afsconf_Reopen(adir);
 }
 
 /* set modtime on file */
-static int
-afsconf_Touch(struct afsconf_dir *adir)
+int
+_afsconf_Touch(struct afsconf_dir *adir)
 {
-    char tbuffer[256];
+    char *cellservDB;
+    int code;
 #ifndef AFS_NT40_ENV
     struct timeval tvp[2];
-#else
-    char *p;
 #endif
 
     adir->timeRead = 0;		/* just in case */
     adir->timeCheck = 0;
 
+    _afsconf_CellServDBPath(adir, &cellservDB);
+    if (cellservDB == NULL)
+	return ENOMEM;
+
 #ifdef AFS_NT40_ENV
-    /* NT client CellServDB has different file name than NT server or Unix */
-
-    if (IsClientConfigDirectory(adir->name)) {
-	if (!afssw_GetClientCellServDBDir(&p)) {
-	    strcompose(tbuffer, sizeof(tbuffer), p, "/",
-		       AFSDIR_CELLSERVDB_FILE_NTCLIENT, NULL);
-	    free(p);
-	} else {
-	    int len = (int)strlen(tbuffer);
-	    if (tbuffer[len - 1] != '\\' && tbuffer[len - 1] != '/') {
-		strncat(tbuffer, "\\", sizeof(tbuffer));
-	    }
-	    strncat(tbuffer, AFSDIR_CELLSERVDB_FILE_NTCLIENT,
-		    sizeof(tbuffer));
-	    tbuffer[sizeof(tbuffer) - 1] = '\0';
-	}
-    } else {
-	strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE,
-		   NULL);
-    }
-
-    return _utime(tbuffer, NULL);
-
+    code = _utime(cellservDB, NULL);
 #else
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE, NULL);
     gettimeofday(&tvp[0], NULL);
     tvp[1] = tvp[0];
-    return utimes(tbuffer, tvp);
+    code = utimes(cellservDB, tvp);
 #endif /* AFS_NT40_ENV */
+    free(cellservDB);
+
+    return code;
 }
 
 struct afsconf_dir *
@@ -446,8 +440,7 @@ afsconf_Open(const char *adir)
 
     LOCK_GLOBAL_MUTEX;
     /* zero structure and fill in name; rest is done by internal routine */
-    tdir = (struct afsconf_dir *)malloc(sizeof(struct afsconf_dir));
-    memset(tdir, 0, sizeof(struct afsconf_dir));
+    tdir = calloc(1, sizeof(struct afsconf_dir));
     tdir->name = strdup(adir);
 
     code = afsconf_OpenInternal(tdir, 0, 0);
@@ -461,6 +454,7 @@ afsconf_Open(const char *adir)
 	    char *home_dir;
 	    afsconf_FILE *fp;
 	    size_t len = 0;
+	    int r;
 
 	    if (!(home_dir = getenv("HOME"))) {
 		/* Our last chance is the "/.AFSCONF" file */
@@ -471,8 +465,8 @@ afsconf_Open(const char *adir)
 	    } else {
 		char *pathname = NULL;
 
-		afs_asprintf(&pathname, "%s/%s", home_dir, ".AFSCONF");
-		if (pathname == NULL)
+		r = asprintf(&pathname, "%s/%s", home_dir, ".AFSCONF");
+		if (r < 0 || pathname == NULL)
 		    goto fail;
 
 		fp = fopen(pathname, "r");
@@ -520,7 +514,8 @@ GetCellUnix(struct afsconf_dir *adir)
     char *start, *p;
     afsconf_FILE *fp;
 
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_THISCELL_FILE, NULL);
+    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_THISCELL_FILE,
+	(char *)NULL);
     fp = fopen(tbuffer, "r");
     if (fp == 0) {
 	return -1;
@@ -549,7 +544,7 @@ GetCellUnix(struct afsconf_dir *adir)
 static int
 GetCellNT(struct afsconf_dir *adir)
 {
-    if (IsClientConfigDirectory(adir->name)) {
+    if (_afsconf_IsClientConfigDirectory(adir->name)) {
 	/* NT client config dir; ThisCell is in registry (no file). */
 	return afssw_GetClientCellName(&adir->cellName);
     } else {
@@ -633,12 +628,17 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
     struct afsconf_aliasentry *curAlias;
     afs_int32 code;
     afs_int32 i;
-    char tbuffer[256], tbuf1[256];
+    char tbuffer[256];
     struct stat tstat;
+    char *cellservDB;
+
 #ifdef AFS_NT40_ENV
     cm_enumCellRegistry_t enumCellRegistry = {0, 0};
 #endif /* AFS_NT40_ENV */
 
+    /* init the keys queue before any call to afsconf_CloseInternal() */
+    _afsconf_InitKeys(adir);
+
     /* figure out the local cell name */
 #ifdef AFS_NT40_ENV
     i = GetCellNT(adir);
@@ -656,48 +656,20 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
     /* now parse the individual lines */
     curEntry = 0;
 
+    _afsconf_CellServDBPath(adir, &cellservDB);
+
 #ifdef AFS_NT40_ENV
-    /* NT client/server have a CellServDB that is the same format as Unix.
-     * However, the NT client uses a different file name
-     */
-    if (IsClientConfigDirectory(adir->name)) {
-	/* NT client config dir */
-	char *p;
-
+    if (_afsconf_IsClientConfigDirectory(adir->name))
         enumCellRegistry.client = 1;
-
-	if (!afssw_GetClientCellServDBDir(&p)) {
-	    strcompose(tbuffer, sizeof(tbuffer), p, "/",
-		       AFSDIR_CELLSERVDB_FILE_NTCLIENT, NULL);
-	    free(p);
-	} else {
-	    int len;
-	    strncpy(tbuffer, adir->name, sizeof(tbuffer));
-	    len = (int)strlen(tbuffer);
-	    if (tbuffer[len - 1] != '\\' && tbuffer[len - 1] != '/') {
-		strncat(tbuffer, "\\", sizeof(tbuffer));
-	    }
-	    strncat(tbuffer, AFSDIR_CELLSERVDB_FILE_NTCLIENT,
-		    sizeof(tbuffer));
-	    tbuffer[sizeof(tbuffer) - 1] = '\0';
-	}
-    } else {
-	/* NT server config dir */
-	strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE,
-		   NULL);
-    }
-#else
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLSERVDB_FILE, NULL);
 #endif /* AFS_NT40_ENV */
 
-    if (!stat(tbuffer, &tstat)) {
+    if (!stat(cellservDB, &tstat)) {
 	adir->timeRead = tstat.st_mtime;
     } else {
 	adir->timeRead = 0;
     }
 
-    strlcpy(tbuf1, tbuffer, sizeof tbuf1);
-    tf = fopen(tbuffer, "r");
+    tf = fopen(cellservDB, "r");
     if (!tf) {
 	return -1;
     }
@@ -730,9 +702,7 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 		adir->entries = curEntry;
 		curEntry = 0;
 	    }
-	    curEntry =
-		(struct afsconf_entry *)malloc(sizeof(struct afsconf_entry));
-	    memset(curEntry, 0, sizeof(struct afsconf_entry));
+	    curEntry = calloc(1, sizeof(struct afsconf_entry));
 	    code =
 		ParseCellLine(tbuffer, curEntry->cellInfo.name, linkedcell);
 	    if (code) {
@@ -773,7 +743,7 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 			*bp = '\0';
 			fprintf(stderr,
 				"Can't properly parse host line \"%s\" in configuration file %s\n",
-				tbuffer, tbuf1);
+				tbuffer, cellservDB);
 		    }
 		    free(curEntry);
 		    fclose(tf);
@@ -784,11 +754,12 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 	    } else {
 		fprintf(stderr,
 			"Too many hosts for cell %s in configuration file %s\n",
-			curEntry->cellInfo.name, tbuf1);
+			curEntry->cellInfo.name, cellservDB);
 	    }
 	}
     }
     fclose(tf);			/* close the file now */
+    free(cellservDB);
 
     /* end the last partially-completed cell */
     if (curEntry) {
@@ -809,7 +780,8 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 #endif /* AFS_NT40_ENV */
 
     /* Read in the alias list */
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLALIAS_FILE, NULL);
+    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_CELLALIAS_FILE,
+	(char *)NULL);
 
     tf = fopen(tbuffer, "r");
     while (tf) {
@@ -840,8 +812,7 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 	    tp++;
 	tp[0] = '\0';
 
-	curAlias = malloc(sizeof(*curAlias));
-	memset(curAlias, 0, sizeof(*curAlias));
+	curAlias = calloc(1, sizeof(*curAlias));
 
 	strlcpy(curAlias->aliasInfo.aliasName, aliasPtr, sizeof curAlias->aliasInfo.aliasName);
 	strlcpy(curAlias->aliasInfo.realName, tbuffer, sizeof curAlias->aliasInfo.realName);
@@ -852,11 +823,15 @@ afsconf_OpenInternal(struct afsconf_dir *adir, char *cell,
 
     if (tf != NULL)
 	fclose(tf);
-    /* now read the fs keys, if possible */
-    adir->keystr = (struct afsconf_keys *)0;
-    afsconf_IntGetKeys(adir);
 
-    return 0;
+    /* now read the fs keys, if possible */
+    code = _afsconf_LoadKeys(adir);
+    if (code) {
+        return code;
+    }
+    code = _afsconf_LoadRealms(adir);
+
+    return code;
 }
 
 /* parse a line of the form
@@ -999,17 +974,17 @@ afsconf_GetExtendedCellInfo(struct afsconf_dir *adir, char *acellName,
 int
 afsconf_LookupServer(const char *service, const char *protocol,
 		     const char *cellName, unsigned short afsdbPort,
-		     int *cellHostAddrs, char cellHostNames[][MAXHOSTCHARS],
+		     afs_uint32 *cellHostAddrs, char cellHostNames[][MAXHOSTCHARS],
 		     unsigned short ports[], unsigned short ipRanks[],
 		     int *numServers, int *ttl, char **arealCellName)
 {
     int code = 0;
+    int r;
     int len;
     unsigned char answer[4096];
     unsigned char *p;
-    char *dotcellname;
+    char *dotcellname = NULL;
     char *realCellName;
-    int cellnamelength, fullnamelength;
     char host[256];
     int server_num = 0;
     int minttl = 0;
@@ -1029,12 +1004,6 @@ afsconf_LookupServer(const char *service, const char *protocol,
     if (strchr(cellName,'.'))
 	pass += 2;
 
-    cellnamelength=strlen(cellName); /* _ ._ . . \0 */
-    fullnamelength=cellnamelength+strlen(protocol)+strlen(IANAname)+6;
-    dotcellname=malloc(fullnamelength);
-    if (!dotcellname)
-	return AFSCONF_NOTFOUND;	/* service not found */
-
 #ifdef HAVE_RES_RETRANSRETRY
     if ((_res.options & RES_INIT) == 0 && res_init() == -1)
       return (0);
@@ -1048,34 +1017,37 @@ afsconf_LookupServer(const char *service, const char *protocol,
 #endif
 
  retryafsdb:
+    r = -1;
     switch (pass) {
     case 0:
 	dnstype = T_SRV;
-	code = snprintf(dotcellname, fullnamelength, "_%s._%s.%s.",
-		 IANAname, protocol, cellName);
+	r = asprintf(&dotcellname, "_%s._%s.%s.", IANAname, protocol, cellName);
 	break;
     case 1:
 	dnstype = T_AFSDB;
-	code = snprintf(dotcellname, fullnamelength, "%s.",
-		 cellName);
+	r = asprintf(&dotcellname, "%s.", cellName);
 	break;
     case 2:
 	dnstype = T_SRV;
-	code = snprintf(dotcellname, fullnamelength, "_%s._%s.%s",
-		 IANAname, protocol, cellName);
+	r = asprintf(&dotcellname, "_%s._%s.%s", IANAname, protocol, cellName);
 	break;
     case 3:
 	dnstype = T_AFSDB;
-	code = snprintf(dotcellname, fullnamelength, "%s",
-		 cellName);
+	r = asprintf(&dotcellname, "%s", cellName);
 	break;
     }
-    if ((code < 0) || (code >= fullnamelength))
+    if (r < 0 || dotcellname == NULL)
 	goto findservererror;
+
     LOCK_GLOBAL_MUTEX;
     len = res_search(dotcellname, C_IN, dnstype, answer, sizeof(answer));
     UNLOCK_GLOBAL_MUTEX;
 
+    if (dotcellname != NULL) {
+	free(dotcellname);
+	dotcellname = NULL;
+    }
+
     if (len < 0) {
 	if (try_init < 1) {
 	    try_init++;
@@ -1141,17 +1113,18 @@ afsconf_LookupServer(const char *service, const char *protocol,
 	    if ((afsdb_type == 1) && (server_num < MAXHOSTSPERCELL) &&
 		/* Do we want to get TTL data for the A record as well? */
 		(he = gethostbyname(host))) {
-		afs_int32 ipaddr;
-		memcpy(&ipaddr, he->h_addr, he->h_length);
-		cellHostAddrs[server_num] = ipaddr;
-		ports[server_num] = afsdbPort;
-		ipRanks[server_num] = 0;
-		strncpy(cellHostNames[server_num], host,
-			sizeof(cellHostNames[server_num]));
-		server_num++;
-
-		if (!minttl || ttl < minttl)
-		    minttl = ttl;
+		if (he->h_addrtype == AF_INET) {
+		    afs_uint32 ipaddr;
+		    memcpy(&ipaddr, he->h_addr, sizeof(ipaddr));
+		    cellHostAddrs[server_num] = ipaddr;
+		    ports[server_num] = afsdbPort;
+		    ipRanks[server_num] = 0;
+		    strncpy(cellHostNames[server_num], host,
+			    sizeof(cellHostNames[server_num]));
+		    server_num++;
+		    if (!minttl || ttl < minttl)
+			minttl = ttl;
+		}
 	    }
 	}
 	if (type == T_SRV) {
@@ -1174,18 +1147,21 @@ afsconf_LookupServer(const char *service, const char *protocol,
 	    if ((server_num < MAXHOSTSPERCELL) &&
 		/* Do we want to get TTL data for the A record as well? */
 		(he = gethostbyname(host))) {
-		afs_int32 ipaddr;
-		memcpy(&ipaddr, he->h_addr, he->h_length);
-		cellHostAddrs[server_num] = ipaddr;
-		ipRanks[server_num] = (p[0] << 8) | p[1];
-		ports[server_num] = htons((p[4] << 8) | p[5]);
-		/* weight = (p[2] << 8) | p[3]; */
-		strncpy(cellHostNames[server_num], host,
-			sizeof(cellHostNames[server_num]));
-		server_num++;
+		if (he->h_addrtype == AF_INET) {
+		    afs_uint32 ipaddr;
 
-		if (!minttl || ttl < minttl)
-		    minttl = ttl;
+		    memcpy(&ipaddr, he->h_addr, sizeof(ipaddr));
+		    cellHostAddrs[server_num] = ipaddr;
+		    ipRanks[server_num] = (p[0] << 8) | p[1];
+		    ports[server_num] = htons((p[4] << 8) | p[5]);
+		    /* weight = (p[2] << 8) | p[3]; */
+		    strncpy(cellHostNames[server_num], host,
+			    sizeof(cellHostNames[server_num]));
+		    server_num++;
+
+		    if (!minttl || ttl < minttl)
+			minttl = ttl;
+		}
 	    }
 	}
 
@@ -1215,7 +1191,6 @@ afsconf_LookupServer(const char *service, const char *protocol,
 findservererror:
     if (code && realCellName)
 	free(realCellName);
-    free(dotcellname);
     return code;
 }
 
@@ -1223,7 +1198,7 @@ int
 afsconf_GetAfsdbInfo(char *acellName, char *aservice,
 		     struct afsconf_cell *acellInfo)
 {
-    afs_int32 cellHostAddrs[AFSMAXCELLHOSTS];
+    afs_uint32 cellHostAddrs[AFSMAXCELLHOSTS];
     char cellHostNames[AFSMAXCELLHOSTS][MAXHOSTCHARS];
     unsigned short ipRanks[AFSMAXCELLHOSTS];
     unsigned short ports[AFSMAXCELLHOSTS];
@@ -1291,7 +1266,7 @@ afsconf_GetAfsdbInfo(char *acellName, char *aservice,
     int tservice = afsconf_FindService(aservice);   /* network byte order */
     const char *ianaName = afsconf_FindIANAName(aservice);
     struct afsconf_entry DNSce;
-    afs_int32 cellHostAddrs[AFSMAXCELLHOSTS];
+    afs_uint32 cellHostAddrs[AFSMAXCELLHOSTS];
     char cellHostNames[AFSMAXCELLHOSTS][MAXHOSTCHARS];
     unsigned short ipRanks[AFSMAXCELLHOSTS];
     unsigned short ports[AFSMAXCELLHOSTS];          /* network byte order */
@@ -1370,7 +1345,7 @@ afsconf_GetCellInfo(struct afsconf_dir *adir, char *acellName, char *aservice,
 
     LOCK_GLOBAL_MUTEX;
     if (adir)
-	afsconf_Check(adir);
+	_afsconf_Check(adir);
     if (acellName) {
 	tcell = acellName;
 	cnLen = (int)(strlen(tcell) + 1);
@@ -1435,7 +1410,7 @@ afsconf_GetCellInfo(struct afsconf_dir *adir, char *acellName, char *aservice,
          * queries from other server queries, only perform gethostbyname()
          * lookup on the specified hostnames for the client CellServDB files.
          */
-        if (IsClientConfigDirectory(adir->name) &&
+        if (_afsconf_IsClientConfigDirectory(adir->name) &&
             !(acellInfo->flags & AFSCONF_CELL_FLAG_DNS_QUERIED)) {
             int j;
             short numServers=0;		                        /*Num active servers for the cell */
@@ -1455,9 +1430,9 @@ afsconf_GetCellInfo(struct afsconf_dir *adir, char *acellName, char *aservice,
                     for (i=0 ; he->h_addr_list[i] && numServers < MAXHOSTSPERCELL; i++) {
                         /* check to see if this is a new address; if so insert it into the list */
                         int k, dup;
+			afs_uint32 addr;
+			memcpy(&addr, he->h_addr_list[i], sizeof(addr));
                         for (k=0, dup=0; !dup && k < numServers; k++) {
-			    afs_uint32 addr;
-			    memcpy(&addr, he->h_addr_list[i], sizeof(addr));
                             if (hostAddr[k].sin_addr.s_addr == addr) {
                                 dup = 1;
 			    }
@@ -1498,36 +1473,68 @@ afsconf_GetCellInfo(struct afsconf_dir *adir, char *acellName, char *aservice,
     }
 }
 
+/**
+ * Get the current localcell name.
+ *
+ * Internal function to get a pointer to the local cell name.
+ * This function must be called with the global afsconf lock held.
+ *
+ * @param[in]  adir    afsconf object
+ * @param[out] aname   address to a char pointer
+ * @param[in]  check   always perform a config check, even if the
+ *                     the AFSCELL name is set.
+ *
+ * @return status
+ *    @retval 0 success
+ *    @retval AFSCONF_NOCELLNAME cannot determine local cell name
+ *
+ * @internal
+ */
 int
-afsconf_GetLocalCell(struct afsconf_dir *adir, char *aname,
-		     afs_int32 alen)
+_afsconf_GetLocalCell(struct afsconf_dir *adir, char **pname, int check)
 {
     static int afsconf_showcell = 0;
     char *afscell_path;
     afs_int32 code = 0;
 
-    LOCK_GLOBAL_MUTEX;
     /*
      * If a cell switch was specified in a command, then it should override the
      * AFSCELL variable.  If a cell was specified, then the afsconf_SawCell flag
      * is set and the cell name in the adir structure is used.
      * Read the AFSCELL var each time: in case it changes (unsetenv AFSCELL).
+     * Optionally, check the configuration, even if using the environment variable.
      */
     if (!afsconf_SawCell && (afscell_path = getenv("AFSCELL"))) {
+	if (check) {
+	    _afsconf_Check(adir);
+	}
 	if (!afsconf_showcell) {
 	    fprintf(stderr, "Note: Operation is performed on cell %s\n",
 		    afscell_path);
 	    afsconf_showcell = 1;
 	}
-	strncpy(aname, afscell_path, alen);
+	*pname = afscell_path;
     } else {
-	afsconf_Check(adir);
+	_afsconf_Check(adir);
 	if (adir->cellName) {
-	    strncpy(aname, adir->cellName, alen);
+	    *pname = adir->cellName;
 	} else
-	    code = AFSCONF_UNKNOWN;
+	    code = AFSCONF_NOCELLNAME;
     }
+    return code;
+}
 
+int
+afsconf_GetLocalCell(struct afsconf_dir *adir, char *aname, afs_int32 alen)
+{
+    afs_int32 code = 0;
+    char *cellname = NULL;
+
+    LOCK_GLOBAL_MUTEX;
+    code = _afsconf_GetLocalCell(adir, &cellname, 0);
+    if (!code && cellname) {
+	strlcpy(aname, cellname, alen);
+    }
     UNLOCK_GLOBAL_MUTEX;
     return (code);
 }
@@ -1566,8 +1573,9 @@ afsconf_CloseInternal(struct afsconf_dir *adir)
 	na = ta->next;
 	free(ta);
     }
-    if (adir->keystr)
-	free(adir->keystr);
+
+    _afsconf_FreeAllKeys(adir);
+    _afsconf_FreeRealms(adir);
 
     /* reinit */
     memset(adir, 0, sizeof(struct afsconf_dir));
@@ -1585,261 +1593,3 @@ afsconf_Reopen(struct afsconf_dir *adir)
     code = afsconf_OpenInternal(adir, 0, 0);
     return code;
 }
-
-/* called during opening of config file */
-int
-afsconf_IntGetKeys(struct afsconf_dir *adir)
-{
-    char tbuffer[256];
-    int fd;
-    struct afsconf_keys *tstr;
-    afs_int32 code;
-
-#ifdef AFS_NT40_ENV
-    /* NT client config dir has no KeyFile; don't risk attempting open
-     * because there might be a random file of this name if dir is shared.
-     */
-    if (IsClientConfigDirectory(adir->name)) {
-	adir->keystr = ((struct afsconf_keys *)
-			malloc(sizeof(struct afsconf_keys)));
-	adir->keystr->nkeys = 0;
-	return 0;
-    }
-#endif /* AFS_NT40_ENV */
-
-    LOCK_GLOBAL_MUTEX;
-    /* compute the key name and other setup */
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_KEY_FILE, NULL);
-    tstr = (struct afsconf_keys *)malloc(sizeof(struct afsconf_keys));
-    adir->keystr = tstr;
-
-    /* read key file */
-    fd = open(tbuffer, O_RDONLY);
-    if (fd < 0) {
-	tstr->nkeys = 0;
-	UNLOCK_GLOBAL_MUTEX;
-	return 0;
-    }
-    code = read(fd, tstr, sizeof(struct afsconf_keys));
-    close(fd);
-    if (code < sizeof(afs_int32)) {
-	tstr->nkeys = 0;
-	UNLOCK_GLOBAL_MUTEX;
-	return 0;
-    }
-
-    /* convert key structure to host order */
-    tstr->nkeys = ntohl(tstr->nkeys);
-
-    if (code < sizeof(afs_int32) + (tstr->nkeys*sizeof(struct afsconf_key))) {
-	tstr->nkeys = 0;
-	UNLOCK_GLOBAL_MUTEX;
-	return 0;
-    }
-
-    for (fd = 0; fd < tstr->nkeys; fd++)
-	tstr->key[fd].kvno = ntohl(tstr->key[fd].kvno);
-
-    UNLOCK_GLOBAL_MUTEX;
-    return 0;
-}
-
-/* get keys structure */
-int
-afsconf_GetKeys(struct afsconf_dir *adir, struct afsconf_keys *astr)
-{
-    afs_int32 code;
-
-    LOCK_GLOBAL_MUTEX;
-    code = afsconf_Check(adir);
-    if (code) {
-	UNLOCK_GLOBAL_MUTEX;
-	return AFSCONF_FAILURE;
-    }
-    memcpy(astr, adir->keystr, sizeof(struct afsconf_keys));
-    UNLOCK_GLOBAL_MUTEX;
-    return 0;
-}
-
-/* get latest key */
-afs_int32
-afsconf_GetLatestKey(struct afsconf_dir * adir, afs_int32 * avno,
-		     struct ktc_encryptionKey *akey)
-{
-    int i;
-    int maxa;
-    struct afsconf_key *tk;
-    afs_int32 best;
-    struct afsconf_key *bestk;
-    afs_int32 code;
-
-    LOCK_GLOBAL_MUTEX;
-    code = afsconf_Check(adir);
-    if (code) {
-	UNLOCK_GLOBAL_MUTEX;
-	return AFSCONF_FAILURE;
-    }
-    maxa = adir->keystr->nkeys;
-
-    best = -1;			/* highest kvno we've seen yet */
-    bestk = (struct afsconf_key *)0;	/* ptr to structure providing best */
-    for (tk = adir->keystr->key, i = 0; i < maxa; i++, tk++) {
-	if (tk->kvno == 999)
-	    continue;		/* skip bcrypt keys */
-	if (tk->kvno > best) {
-	    best = tk->kvno;
-	    bestk = tk;
-	}
-    }
-    if (bestk) {		/* found any  */
-	if (akey)
-	    memcpy(akey, bestk->key, 8);	/* copy out latest key */
-	if (avno)
-	    *avno = bestk->kvno;	/* and kvno to caller */
-	UNLOCK_GLOBAL_MUTEX;
-	return 0;
-    }
-    UNLOCK_GLOBAL_MUTEX;
-    return AFSCONF_NOTFOUND;	/* didn't find any keys */
-}
-
-/* get a particular key */
-int
-afsconf_GetKey(void *rock, int avno, struct ktc_encryptionKey *akey)
-{
-    struct afsconf_dir *adir = (struct afsconf_dir *) rock;
-    int i, maxa;
-    struct afsconf_key *tk;
-    afs_int32 code;
-
-    LOCK_GLOBAL_MUTEX;
-    code = afsconf_Check(adir);
-    if (code) {
-	UNLOCK_GLOBAL_MUTEX;
-	return AFSCONF_FAILURE;
-    }
-    maxa = adir->keystr->nkeys;
-
-    for (tk = adir->keystr->key, i = 0; i < maxa; i++, tk++) {
-	if (tk->kvno == avno) {
-	    memcpy(akey, tk->key, 8);
-	    UNLOCK_GLOBAL_MUTEX;
-	    return 0;
-	}
-    }
-
-    UNLOCK_GLOBAL_MUTEX;
-    return AFSCONF_NOTFOUND;
-}
-
-/* save the key structure in the appropriate file */
-static int
-SaveKeys(struct afsconf_dir *adir)
-{
-    struct afsconf_keys tkeys;
-    int fd;
-    afs_int32 i;
-    char tbuffer[256];
-
-    memcpy(&tkeys, adir->keystr, sizeof(struct afsconf_keys));
-
-    /* convert it to net byte order */
-    for (i = 0; i < tkeys.nkeys; i++)
-	tkeys.key[i].kvno = htonl(tkeys.key[i].kvno);
-    tkeys.nkeys = htonl(tkeys.nkeys);
-
-    /* rewrite keys file */
-    strcompose(tbuffer, 256, adir->name, "/", AFSDIR_KEY_FILE, NULL);
-    fd = open(tbuffer, O_RDWR | O_CREAT | O_TRUNC, 0600);
-    if (fd < 0)
-	return AFSCONF_FAILURE;
-    i = write(fd, &tkeys, sizeof(tkeys));
-    if (i != sizeof(tkeys)) {
-	close(fd);
-	return AFSCONF_FAILURE;
-    }
-    if (close(fd) < 0)
-	return AFSCONF_FAILURE;
-    return 0;
-}
-
-int
-afsconf_AddKey(struct afsconf_dir *adir, afs_int32 akvno, char akey[8],
-	       afs_int32 overwrite)
-{
-    struct afsconf_keys *tk;
-    struct afsconf_key *tkey;
-    afs_int32 i;
-    int foundSlot;
-
-    LOCK_GLOBAL_MUTEX;
-    tk = adir->keystr;
-
-    if (akvno != 999) {
-	if (akvno < 0 || akvno > 255) {
-	    UNLOCK_GLOBAL_MUTEX;
-	    return ERANGE;
-	}
-    }
-    foundSlot = 0;
-    for (i = 0, tkey = tk->key; i < tk->nkeys; i++, tkey++) {
-	if (tkey->kvno == akvno) {
-	    if (!overwrite) {
-		UNLOCK_GLOBAL_MUTEX;
-		return AFSCONF_KEYINUSE;
-	    }
-	    foundSlot = 1;
-	    break;
-	}
-    }
-    if (!foundSlot) {
-	if (tk->nkeys >= AFSCONF_MAXKEYS) {
-	    UNLOCK_GLOBAL_MUTEX;
-	    return AFSCONF_FULL;
-	}
-	tkey = &tk->key[tk->nkeys++];
-    }
-    tkey->kvno = akvno;
-    memcpy(tkey->key, akey, 8);
-    i = SaveKeys(adir);
-    afsconf_Touch(adir);
-    UNLOCK_GLOBAL_MUTEX;
-    return i;
-}
-
-/* this proc works by sliding the other guys down, rather than using a funny
-    kvno value, so that callers can count on getting a good key in key[0].
-*/
-int
-afsconf_DeleteKey(struct afsconf_dir *adir, afs_int32 akvno)
-{
-    struct afsconf_keys *tk;
-    struct afsconf_key *tkey;
-    int i;
-    int foundFlag = 0;
-
-    LOCK_GLOBAL_MUTEX;
-    tk = adir->keystr;
-
-    for (i = 0, tkey = tk->key; i < tk->nkeys; i++, tkey++) {
-	if (tkey->kvno == akvno) {
-	    foundFlag = 1;
-	    break;
-	}
-    }
-    if (!foundFlag) {
-	UNLOCK_GLOBAL_MUTEX;
-	return AFSCONF_NOTFOUND;
-    }
-
-    /* otherwise slide the others down.  i and tkey point at the guy to delete */
-    for (; i < tk->nkeys - 1; i++, tkey++) {
-	tkey->kvno = (tkey + 1)->kvno;
-	memcpy(tkey->key, (tkey + 1)->key, 8);
-    }
-    tk->nkeys--;
-    i = SaveKeys(adir);
-    afsconf_Touch(adir);
-    UNLOCK_GLOBAL_MUTEX;
-    return i;
-}
