@@ -9,22 +9,15 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <errno.h>
+
+#include <roken.h>
+
 #include <afs/budb_client.h>
 #include <afs/cmd.h>
 #include <afs/com_err.h>
 #include <afs/bubasics.h>
+
 #include "bc.h"
 #include "error_macros.h"
 #include "bucoord_internal.h"
@@ -491,10 +484,8 @@ bc_ParseVolumeSet(void)
 	     * the info just read placing it at the head of its queue in the
 	     * global configuration structure.
 	     */
-	    tvs = (struct bc_volumeSet *)malloc(sizeof(struct bc_volumeSet));
-	    memset(tvs, 0, sizeof(*tvs));
-	    tvs->name = (char *)malloc(strlen(vsname) + 1);
-	    strcpy(tvs->name, vsname);
+	    tvs = calloc(1, sizeof(struct bc_volumeSet));
+	    tvs->name = strdup(vsname);
 
 	    /* append to the end */
 	    for (ppvs = &bc_globalConfig->vset, pvs = *ppvs; pvs;
@@ -519,14 +510,12 @@ bc_ParseVolumeSet(void)
 	     * spec record, then get the rest of the information regarding
 	     * the host, and stuff everything into place.
 	     */
-	    tve = (struct bc_volumeEntry *)
-		malloc(sizeof(struct bc_volumeEntry));
+	    tve = calloc(1, sizeof(struct bc_volumeEntry));
 	    if (!tve) {
 		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec record!");
 		return (-1);
 	    }
-	    memset(tve, 0, sizeof(*tve));
 	    if (bc_ParseHost(serverName, &(tve->server)))
 		afs_com_err(whoami, 0, "Can't get required info on host '%s'",
 			serverName);
@@ -534,32 +523,29 @@ bc_ParseVolumeSet(void)
 	    /* The above code has filled in the server sockaddr, now fill in
 	     * the rest of the fields.
 	     */
-	    tve->serverName = (char *)malloc(strlen(serverName) + 1);
+	    tve->serverName = strdup(serverName);
 	    if (!tve->serverName) {
 		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec server name field!");
 		return (-1);
 	    }
-	    strcpy(tve->serverName, serverName);
-	    tve->partname = (char *)malloc(strlen(partName) + 1);
+	    tve->partname = strdup(partName);
 	    if (!tve->partname) {
 		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec partition pattern field!");
 		return (-1);
 	    }
-	    strcpy(tve->partname, partName);
 	    code = bc_GetPartitionID(partName, &tve->partition);
 	    if (code) {
 		afs_com_err(whoami, 0, "Can't parse partition '%s'", partName);
 		return -1;
 	    }
-	    tp = (char *)malloc(strlen(vsname) + 1);
+	    tp = strdup(vsname);
 	    if (!tp) {
 		afs_com_err(whoami, 0,
 			"Can't malloc() a new volume spec volume pattern field!");
 		return (-1);
 	    }
-	    strcpy(tp, vsname);
 	    tve->name = tp;
 
 	    /* Now, thread it onto the list of other volume spec entries for
