@@ -10,55 +10,54 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wimplicit-function-declaration"
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#else
-#include <netinet/in.h>
-#include <sys/time.h>
 #endif
-#include <afs/procmgmt.h>
+
+#include <ctype.h>
+
+#include <rx/rx.h>
+#include <rx/rx_globals.h>
+#include <rx/rxkad.h>
 #include <rx/xdr.h>
+
 #include <afs/afsint.h>
-#include <stdio.h>
-#include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
-#include <string.h>
 #include <afs/vlserver.h>
 #include <lwp.h>
 #include <lock.h>
 #include <afs/afsutil.h>
-#include <rx/rx.h>
-#include <rx/rx_globals.h>
-#include <rx/rxkad.h>
 #include <afs/cellconfig.h>
 #include <afs/keys.h>
 #include <afs/volser.h>
 #include <ubik.h>
+#include <afs/audit.h>
 #include <afs/com_err.h>
-#include <errno.h>
 #include <afs/cmd.h>
 #include <afs/tcdata.h>
 #include <afs/bubasics.h>
-#include <ctype.h>
-#include "error_macros.h"
 #include <afs/budb_errs.h>
 #include <afs/budb_client.h>
 #include <afs/bucoord_prototypes.h>
-#include "afs/butx.h"
+#include <afs/butx.h>
+#include <afs/kautils.h>
+#include <afs/bc.h>
+
+#include "error_macros.h"
 #define XBSA_TCMAIN
 #include "butc_xbsa.h"
 #include "butc_prototypes.h"
 #include "butc_internal.h"
-#include <afs/kautils.h>
-#include <afs/bc.h>
 
 #define N_SECURITY_OBJECTS 3
 #define ERRCODE_RANGE 8		/* from error_table.h */
@@ -107,18 +106,12 @@ afs_int32 BufferSize;		/* Size in B stored for data */
 char *centralLogFile;
 afs_int32 lastLog;		/* Log last pass info */
 int rxBind = 0;
+struct afsconf_dir *butc_confdir;
+int allow_unauth = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
 
-/* dummy routine for the audit work.  It should do nothing since audits */
-/* occur at the server level and bos is not a server. */
-int
-osi_audit(void)
-{
-    return 0;
-}
-
 static afs_int32
 SafeATOL(char *anum)
 {
@@ -291,10 +284,9 @@ stringNowReplace(char *logFile, char *deviceName)
 	*pos = '_';
     strcat(logFile, deviceName);
     /* now put back deviceName to the way it was */
-    if (mvFlag) {
-	mvFlag = 0;
+    if (mvFlag)
 	deviceName -= devPrefLen;
-    }
+
     strcpy(deviceName, storeDevice);
 
     return (0);
@@ -517,8 +509,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		continue;
 	    }
 
-	    opencallout = (char *)malloc(strlen(value) + 1);
-	    strcpy(opencallout, value);
+	    opencallout = strdup(value);
 	    printf("Tape mount callout routine is %s\n", opencallout);
 	}
 
@@ -530,8 +521,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		continue;
 	    }
 
-	    closecallout = (char *)malloc(strlen(value) + 1);
-	    strcpy(closecallout, value);
+	    closecallout = strdup(value);
 	    printf("Tape unmount callout routine is %s\n", closecallout);
 	}
 
@@ -664,8 +654,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		     cmd);
 		continue;
 	    }
-	    xbsaObjectOwner = malloc(strlen(value) + 1);
-	    strcpy(xbsaObjectOwner, value);
+	    xbsaObjectOwner = strdup(value);
 	    printf("XBSA node is %s\n", xbsaObjectOwner);
 	}
 
@@ -676,8 +665,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		     cmd);
 		continue;
 	    }
-	    adsmServerName = malloc(strlen(value) + 1);
-	    strcpy(adsmServerName, value);
+	    adsmServerName = strdup(value);
 	    printf("XBSA server is %s\n", adsmServerName);
 	}
 
@@ -695,8 +683,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		continue;
 	    }
 
-	    xbsaSecToken = malloc(strlen(value) + 1);
-	    strcpy(xbsaSecToken, value);
+	    xbsaSecToken = strdup(value);
 	    printf("XBSA Password has been read\n");
 	}
 
@@ -739,8 +726,7 @@ GetConfigParams(char *filename, afs_int32 port)
 		     cmd);
 		continue;
 	    }
-	    xbsalGName = malloc(strlen(value) + 1);
-	    strcpy(xbsalGName, value);
+	    xbsalGName = strdup(value);
 	    printf("XBSA management class is %s\n", xbsalGName);
 	}
 #endif
@@ -774,8 +760,7 @@ GetConfigParams(char *filename, afs_int32 port)
 	}
 
 	else if (!strcmp(cmd, "CENTRALLOG")) {
-	    centralLogFile = malloc(strlen(value) + 1);
-	    strcpy(centralLogFile, value);
+	    centralLogFile = strdup(value);
 	    printf("Central log file is %s\n", centralLogFile);
 	}
 
@@ -841,11 +826,26 @@ xbsa_shutdown(int x)
 }
 #endif
 
+static int
+tc_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
+{
+    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
+    afs_int32 islocal = 0;	/* default to no */
+    int code;
+
+    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
+    if (code) {
+	TLog(0, ("Failed local realm check; code=%d, name=%s, inst=%s, cell=%s\n",
+		 code, name, inst, cell));
+    }
+    return islocal;
+}
+
 static int
 WorkerBee(struct cmd_syndesc *as, void *arock)
 {
-    afs_int32 code;
-    struct rx_securityClass *(securityObjects[1]);
+    afs_int32 code, numClasses;
+    struct rx_securityClass *(nullObjects[1]), **secObjs, **allObjs;
     struct rx_service *service;
     time_t tokenExpires;
     char cellName[64];
@@ -860,6 +860,8 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     PROCESS dbWatcherPid;
 #endif
     afs_uint32 host = htonl(INADDR_ANY);
+    char *auditFileName = NULL;
+    char *auditInterface = NULL;
 
     debugLevel = 0;
 
@@ -915,9 +917,7 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     }
 
     if (as->parms[6].items) {	/* -restoretofile */
-	int s = strlen(as->parms[6].items->data);
-	restoretofile = malloc(s + 1);
-	strncpy(restoretofile, as->parms[6].items->data, s + 1);
+	restoretofile = strdup(as->parms[6].items->data);
 	printf("Restore to file '%s'\n", restoretofile);
     }
 
@@ -967,7 +967,7 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	struct stat sbuf;
 	afs_int32 statcode;
 #ifndef AFS_NT40_ENV
-	char path[AFSDIR_PATH_MAX];
+	char *path;
 #endif
 
 	statcode = stat(centralLogFile, &sbuf);
@@ -979,7 +979,8 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	}
 #ifndef AFS_NT40_ENV
 	/* Make sure it is not in AFS, has to have been created first */
-	if (!realpath(centralLogFile, path)) {
+	path = malloc(AFSDIR_PATH_MAX);
+	if (path == NULL || !realpath(centralLogFile, path)) {
 	    fprintf(stderr,
 		    "Warning: can't determine real path of '%s' (%d)\n",
 		    centralLogFile, errno);
@@ -990,6 +991,7 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 		exit(1);
 	    }
 	}
+	free(path);
 #endif
 
 	/* Write header if created it */
@@ -1005,6 +1007,30 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	}
     }
 
+    /* Open the configuration directory */
+    butc_confdir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
+    if (butc_confdir == NULL) {
+	TLog(0, "Failed to open server configuration directory");
+	exit(1);
+    }
+
+    /* Start auditing */
+    osi_audit_init();
+    if (as->parms[9].items) {
+	auditFileName = as->parms[9].items->data;
+    }
+    if (auditFileName != NULL)
+	osi_audit_file(auditFileName);
+    if (as->parms[10].items) {
+	auditInterface = as->parms[10].items->data;
+	if (osi_audit_interface(auditInterface)) {
+	    TLog(0, "Invalid audit interface '%s'\n", auditInterface);
+	    exit(1);
+	}
+    }
+    osi_audit(TC_StartEvent, 0, AUD_END);
+    osi_audit_set_user_check(butc_confdir, tc_IsLocalRealmMatch);
+
     if (as->parms[1].items) {
 	debugLevel = SafeATOL(as->parms[1].items->data);
 	if (debugLevel == -1) {
@@ -1045,16 +1071,23 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     localauth = (as->parms[5].items ? 1 : 0);
     rxBind = (as->parms[8].items ? 1 : 0);
+    allow_unauth = (as->parms[11].items ? 1 : 0);
+
+    if (!allow_unauth && !localauth) {
+	const char *errstr = "Neither -localauth nor -allow_unauthenticated was provided; refusing to start in unintended insecure configuration\n";
+	TLog(0, "%s", (char *)errstr);
+	exit(1);
+    }
 
     if (rxBind) {
         afs_int32 ccode;
         if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+            ccode = afsconf_ParseNetFiles(SHostAddrs, NULL, NULL,
+                                          ADDRSPERSITE, reason,
+                                          AFSDIR_SERVER_NETINFO_FILEPATH,
+                                          AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
@@ -1083,26 +1116,54 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     /*initialize the dumpNode list */
     InitNodeList(portOffset);
 
-    deviceLatch =
-	(struct deviceSyncNode *)(malloc(sizeof(struct deviceSyncNode)));
+    deviceLatch = malloc(sizeof(struct deviceSyncNode));
     Lock_Init(&(deviceLatch->lock));
     deviceLatch->flags = 0;
 
     /* initialize database support, volume support, and logs */
 
-    /* Create a single security object, in this case the null security
-     * object, for unauthenticated connections, which will be used to control
-     * security on connections made to this server
+    /*
+     * Create security objects for the Rx server functionality.  Historically
+     * this was a single rxnull security object, since the tape controller was
+     * run by an operator that had local access to the tape device and some
+     * administrative privilege in the cell (to be able to perform volume-level
+     * accesses), but on a machine that was not necessarily trusted to hold the
+     * cell-wide key.
+     *
+     * Such a configuration is, of course, insecure because anyone can make
+     * inbound RPCs and manipulate the database, including creating bogus
+     * dumps and restoring them!  Additionally, in modern usage, butc is
+     * frequently run with -localauth to authenticate its outbound connections
+     * to the volservers and budb with the cell-wide key, in which case the
+     * cell-wide key is present and could be used to authenticate incoming
+     * connections as well.
+     *
+     * If -localauth is in use, create the full barrage of server security
+     * objects, including rxkad, so that inbound connections can be verified
+     * to only be made by authenticated clients.  Otherwise, only the rxnull
+     * class is in use with a single server security object.  Note that butc
+     * will refuse to start in this configuration unless the
+     * "-allow_unauthenticated" flag is provided, indicating that the operator
+     * has ensured that incoming connections are appropriately restricted by
+     * firewall configuration or network topology.
      */
 
-    securityObjects[0] = rxnull_NewServerSecurityObject();
-    if (!securityObjects[0]) {
-	TLog(0, "rxnull_NewServerSecurityObject");
-	exit(1);
+    if (allow_unauth) {
+	nullObjects[RX_SECIDX_NULL] = rxnull_NewServerSecurityObject();
+	if (!nullObjects[RX_SECIDX_NULL]) {
+	    TLog(0, "rxnull_NewServerSecurityObject");
+	    exit(1);
+	}
+	numClasses = 1;
+	secObjs = nullObjects;
+    } else {
+	/* Must be -localauth, so the cell keys are available. */
+	afsconf_BuildServerSecurityObjects(butc_confdir, &allObjs, &numClasses);
+	secObjs = allObjs;
     }
 
     service =
-	rx_NewServiceHost(host, 0, 1, "BUTC", securityObjects, 1, TC_ExecuteRequest);
+	rx_NewServiceHost(host, 0, 1, "BUTC", secObjs, numClasses, TC_ExecuteRequest);
     if (!service) {
 	TLog(0, "rx_NewService");
 	exit(1);
@@ -1189,7 +1250,7 @@ main(int argc, char **argv)
 
     setlinebuf(stdout);
 
-    ts = cmd_CreateSyntax(NULL, WorkerBee, NULL, "tape coordinator");
+    ts = cmd_CreateSyntax(NULL, WorkerBee, NULL, 0, "tape coordinator");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "port offset");
     cmd_AddParm(ts, "-debuglevel", CMD_SINGLE, CMD_OPTIONAL, "0 | 1 | 2");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cell name");
@@ -1205,6 +1266,11 @@ main(int argc, char **argv)
 		"Force multiple XBSA server support");
     cmd_AddParm(ts, "-rxbind", CMD_FLAG, CMD_OPTIONAL,
 		"bind Rx socket");
+    cmd_AddParm(ts, "-auditlog", CMD_SINGLE, CMD_OPTIONAL, "location of audit log");
+    cmd_AddParm(ts, "-audit-interface", CMD_SINGLE, CMD_OPTIONAL,
+		"interface to use for audit logging");
+    cmd_AddParm(ts, "-allow_unauthenticated", CMD_FLAG, CMD_OPTIONAL,
+		"allow unauthenticated inbound RPCs (requires firewalling)");
 
     /* Initialize dirpaths */
     if (!(initAFSDirPath() & AFSDIR_SERVER_PATHS_OK)) {
@@ -1217,32 +1283,31 @@ main(int argc, char **argv)
 
     /* setup the file paths */
     strcompose(eFile, AFSDIR_PATH_MAX, AFSDIR_SERVER_BACKUP_DIRPATH, "/",
-	       TE_PREFIX, NULL);
+	       TE_PREFIX, (char *)NULL);
     strcompose(lFile, AFSDIR_PATH_MAX, AFSDIR_SERVER_BACKUP_DIRPATH, "/",
-	       TL_PREFIX, NULL);
+	       TL_PREFIX, (char *)NULL);
     strcompose(pFile, AFSDIR_PATH_MAX, AFSDIR_SERVER_BACKUP_DIRPATH, "/",
-	       CFG_PREFIX, NULL);
+	       CFG_PREFIX, (char *)NULL);
     strcpy(tapeConfigFile, AFSDIR_SERVER_TAPECONFIG_FILEPATH);
 
     /* special case "no args" case since cmd_dispatch gives help message
      * instead
      */
     if (argc == 1) {
-	ts = (struct cmd_syndesc *)malloc(sizeof(struct cmd_syndesc));
-	memset(ts, 0, sizeof(*ts));
+	ts = calloc(1, sizeof(struct cmd_syndesc));
 
-	ti = (struct cmd_item *)malloc(sizeof(struct cmd_item));
+	ti = malloc(sizeof(struct cmd_item));
 	ti->next = 0;
 	ti->data = "0";
 	ts->parms[0].items = ti;
-	ti = (struct cmd_item *)malloc(sizeof(struct cmd_item));
+	ti = malloc(sizeof(struct cmd_item));
 	ti->next = 0;
 	ti->data = "0";
 	ts->parms[1].items = ti;
-	ts->parms[2].items = (struct cmd_item *)NULL;
-	ts->parms[3].items = (struct cmd_item *)NULL;
-	ts->parms[4].items = (struct cmd_item *)NULL;
-	ts->parms[5].items = (struct cmd_item *)NULL;
+	ts->parms[2].items = NULL;
+	ts->parms[3].items = NULL;
+	ts->parms[4].items = NULL;
+	ts->parms[5].items = NULL;
 	return WorkerBee(ts, NULL);
     } else
 	return cmd_Dispatch(argc, argv);
