@@ -166,10 +166,11 @@
 
 	//get link configuration
 	NSData *prefData = (NSData*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_LINK_CONFIGURATION, (CFStringRef)kAfsCommanderID, kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
-	linkConfiguration = (NSMutableDictionary*)[NSPropertyListSerialization propertyListFromData:prefData
-																			   mutabilityOption:NSPropertyListMutableContainers
-																						 format:nil
-																			   errorDescription:nil];
+	NSDictionary *linkDict = (NSMutableDictionary*)[NSPropertyListSerialization propertyListFromData:prefData
+										    mutabilityOption:NSPropertyListMutableContainers
+										    format:nil
+										    errorDescription:nil];
+	linkConfiguration = [linkDict mutableCopy];
 	
 	//get link enabled status
 	NSNumber *linkEnabledStatus =  (NSNumber*)CFPreferencesCopyValue((CFStringRef)PREFERENCE_USE_LINK, (CFStringRef)kAfsCommanderID, kCFPreferencesCurrentUser, kCFPreferencesAnyHost);
